##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te

from atproto_client.models import base, string_formats


class Data(base.DataModelBase):
    """Input data model for :obj:`com.atproto.server.deactivateAccount`."""

    delete_after: t.Optional[string_formats.DateTime] = (
        None  #: A recommendation to server as to how long they should hold onto the deactivated account before deleting.
    )


class DataDict(t.TypedDict):
    delete_after: te.NotRequired[
        t.Optional[string_formats.DateTime]
    ]  #: A recommendation to server as to how long they should hold onto the deactivated account before deleting.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from pydantic import Field

from atproto_client.models import base, string_formats


class Set(base.ModelBase):
    """Definition model for :obj:`tools.ozone.set.defs`."""

    name: str = Field(min_length=3, max_length=128)  #: Name.
    description: t.Optional[str] = Field(default=None, max_length=10240)  #: Description.

    py_type: t.Literal['tools.ozone.set.defs#set'] = Field(
        default='tools.ozone.set.defs#set', alias='$type', frozen=True
    )


class SetView(base.ModelBase):
    """Definition model for :obj:`tools.ozone.set.defs`."""

    created_at: string_formats.DateTime  #: Created at.
    name: str = Field(min_length=3, max_length=128)  #: Name.
    set_size: int  #: Set size.
    updated_at: string_formats.DateTime  #: Updated at.
    description: t.Optional[str] = Field(default=None, max_length=10240)  #: Description.

    py_type: t.Literal['tools.ozone.set.defs#setView'] = Field(
        default='tools.ozone.set.defs#setView', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
    from atproto_client.models.unknown_type import UnknownType
from atproto_client.models import base


class ListViewBasic(base.ModelBase):
    """Definition model for :obj:`app.bsky.graph.defs`."""

    cid: string_formats.Cid  #: Cid.
    name: str = Field(min_length=1, max_length=64)  #: Name.
    purpose: 'models.AppBskyGraphDefs.ListPurpose'  #: Purpose.
    uri: string_formats.AtUri  #: Uri.
    avatar: t.Optional[string_formats.Uri] = None  #: Avatar.
    indexed_at: t.Optional[string_formats.DateTime] = None  #: Indexed at.
    labels: t.Optional[t.List['models.ComAtprotoLabelDefs.Label']] = None  #: Labels.
    list_item_count: t.Optional[int] = Field(default=None, ge=0)  #: List item count.
    viewer: t.Optional['models.AppBskyGraphDefs.ListViewerState'] = None  #: Viewer.

    py_type: t.Literal['app.bsky.graph.defs#listViewBasic'] = Field(
        default='app.bsky.graph.defs#listViewBasic', alias='$type', frozen=True
    )


class ListView(base.ModelBase):
    """Definition model for :obj:`app.bsky.graph.defs`."""

    cid: string_formats.Cid  #: Cid.
    creator: 'models.AppBskyActorDefs.ProfileView'  #: Creator.
    indexed_at: string_formats.DateTime  #: Indexed at.
    name: str = Field(min_length=1, max_length=64)  #: Name.
    purpose: 'models.AppBskyGraphDefs.ListPurpose'  #: Purpose.
    uri: string_formats.AtUri  #: Uri.
    avatar: t.Optional[string_formats.Uri] = None  #: Avatar.
    description: t.Optional[str] = Field(default=None, max_length=3000)  #: Description.
    description_facets: t.Optional[t.List['models.AppBskyRichtextFacet.Main']] = None  #: Description facets.
    labels: t.Optional[t.List['models.ComAtprotoLabelDefs.Label']] = None  #: Labels.
    list_item_count: t.Optional[int] = Field(default=None, ge=0)  #: List item count.
    viewer: t.Optional['models.AppBskyGraphDefs.ListViewerState'] = None  #: Viewer.

    py_type: t.Literal['app.bsky.graph.defs#listView'] = Field(
        default='app.bsky.graph.defs#listView', alias='$type', frozen=True
    )


class ListItemView(base.ModelBase):
    """Definition model for :obj:`app.bsky.graph.defs`."""

    subject: 'models.AppBskyActorDefs.ProfileView'  #: Subject.
    uri: string_formats.AtUri  #: Uri.

    py_type: t.Literal['app.bsky.graph.defs#listItemView'] = Field(
        default='app.bsky.graph.defs#listItemView', alias='$type', frozen=True
    )


class StarterPackView(base.ModelBase):
    """Definition model for :obj:`app.bsky.graph.defs`."""

    cid: string_formats.Cid  #: Cid.
    creator: 'models.AppBskyActorDefs.ProfileViewBasic'  #: Creator.
    indexed_at: string_formats.DateTime  #: Indexed at.
    record: 'UnknownType'  #: Record.
    uri: string_formats.AtUri  #: Uri.
    feeds: t.Optional[t.List['models.AppBskyFeedDefs.GeneratorView']] = Field(default=None, max_length=3)  #: Feeds.
    joined_all_time_count: t.Optional[int] = Field(default=None, ge=0)  #: Joined all time count.
    joined_week_count: t.Optional[int] = Field(default=None, ge=0)  #: Joined week count.
    labels: t.Optional[t.List['models.ComAtprotoLabelDefs.Label']] = None  #: Labels.
    list: t.Optional['models.AppBskyGraphDefs.ListViewBasic'] = None  #: List.
    list_items_sample: t.Optional[t.List['models.AppBskyGraphDefs.ListItemView']] = Field(
        default=None, max_length=12
    )  #: List items sample.

    py_type: t.Literal['app.bsky.graph.defs#starterPackView'] = Field(
        default='app.bsky.graph.defs#starterPackView', alias='$type', frozen=True
    )


class StarterPackViewBasic(base.ModelBase):
    """Definition model for :obj:`app.bsky.graph.defs`."""

    cid: string_formats.Cid  #: Cid.
    creator: 'models.AppBskyActorDefs.ProfileViewBasic'  #: Creator.
    indexed_at: string_formats.DateTime  #: Indexed at.
    record: 'UnknownType'  #: Record.
    uri: string_formats.AtUri  #: Uri.
    joined_all_time_count: t.Optional[int] = Field(default=None, ge=0)  #: Joined all time count.
    joined_week_count: t.Optional[int] = Field(default=None, ge=0)  #: Joined week count.
    labels: t.Optional[t.List['models.ComAtprotoLabelDefs.Label']] = None  #: Labels.
    list_item_count: t.Optional[int] = Field(default=None, ge=0)  #: List item count.

    py_type: t.Literal['app.bsky.graph.defs#starterPackViewBasic'] = Field(
        default='app.bsky.graph.defs#starterPackViewBasic', alias='$type', frozen=True
    )


ListPurpose = t.Union[
    'models.AppBskyGraphDefs.Modlist',
    'models.AppBskyGraphDefs.Curatelist',
    'models.AppBskyGraphDefs.Referencelist',
    str,
]  #: List purpose

Modlist = t.Literal[
    'app.bsky.graph.defs#modlist'
]  #: A list of actors to apply an aggregate moderation action (mute/block) on.

Curatelist = t.Literal[
    'app.bsky.graph.defs#curatelist'
]  #: A list of actors used for curation purposes such as list feeds or interaction gating.

Referencelist = t.Literal[
    'app.bsky.graph.defs#referencelist'
]  #: A list of actors used for only for reference purposes such as within a starter pack.


class ListViewerState(base.ModelBase):
    """Definition model for :obj:`app.bsky.graph.defs`."""

    blocked: t.Optional[string_formats.AtUri] = None  #: Blocked.
    muted: t.Optional[bool] = None  #: Muted.

    py_type: t.Literal['app.bsky.graph.defs#listViewerState'] = Field(
        default='app.bsky.graph.defs#listViewerState', alias='$type', frozen=True
    )


class NotFoundActor(base.ModelBase):
    """Definition model for :obj:`app.bsky.graph.defs`. indicates that a handle or DID could not be resolved."""

    actor: string_formats.AtIdentifier  #: Actor.
    not_found: bool = Field(frozen=True)  #: Not found.

    py_type: t.Literal['app.bsky.graph.defs#notFoundActor'] = Field(
        default='app.bsky.graph.defs#notFoundActor', alias='$type', frozen=True
    )


class Relationship(base.ModelBase):
    """Definition model for :obj:`app.bsky.graph.defs`. lists the bi-directional graph relationships between one actor (not indicated in the object), and the target actors (the DID included in the object)."""

    did: string_formats.Did  #: Did.
    followed_by: t.Optional[string_formats.AtUri] = (
        None  #: If the actor is followed by this DID, contains the AT-URI of the follow record.
    )
    following: t.Optional[string_formats.AtUri] = (
        None  #: If the actor follows this DID, this is the AT-URI of the follow record.
    )

    py_type: t.Literal['app.bsky.graph.defs#relationship'] = Field(
        default='app.bsky.graph.defs#relationship', alias='$type', frozen=True
    )



import typing as t
from dataclasses import dataclass
from importlib import metadata as importlib_metadata

import httpx
import typing_extensions as te
from pydantic_core import from_json

from atproto_client import exceptions
from atproto_client.models.common import XrpcError
from atproto_client.models.utils import get_or_create, load_json

try:
    _ATPROTO_SDK_VERSION = importlib_metadata.version('atproto')
except importlib_metadata.PackageNotFoundError:
    _ATPROTO_SDK_VERSION = 'unknown'

_ATPROTO_SDK_USER_AGENT = f'atproto/{_ATPROTO_SDK_VERSION} Python SDK (atproto.blue)'


@dataclass
class Response:
    success: bool
    status_code: int
    content: t.Optional[t.Union[t.Dict[str, t.Any], bytes, 'XrpcError']]
    headers: t.Dict[str, t.Any]


_NormalizedHeaders = t.Dict[str, t.Tuple[str, str]]


def _normalize_headers(headers: t.Dict[str, str]) -> _NormalizedHeaders:
    """Normalize headers by converting keys to lowercase.

    Args:
        headers: Headers dictionary with the original case.

    Returns:
        Dictionary with lowercase keys mapped to tuples of (original_key, value).
    """
    return {key.lower(): (key, value) for key, value in headers.items()}


def _denormalize_headers(headers: _NormalizedHeaders) -> t.Dict[str, str]:
    """Denormalize headers by converting keys back to their original case.

    Args:
        headers: Headers dictionary with lowercase keys mapped to tuples of (original_key, value).

    Returns:
        Dictionary with original keys and values.
    """
    return {original_key: value for _, (original_key, value) in headers.items()}


def _convert_headers_to_dict(headers: httpx.Headers) -> t.Dict[str, str]:
    """Convert custom case-insensitive multi-dict of HTTPX to pure dict with lowercase keys.

    Note:
        Concatenate headers into a single comma separated value when a key occurs multiple times.
    """
    return {key.lower(): value for key, value in headers.items()}


def _parse_response(response: httpx.Response) -> Response:
    content = response.content
    content_type = response.headers.get('content-type')
    if content_type and 'application/json' in content_type.lower():
        content = from_json(response.content)

    return Response(
        success=True,
        status_code=response.status_code,
        content=content,
        headers=_convert_headers_to_dict(response.headers),
    )


def _handle_request_errors(exception: Exception) -> None:
    try:
        raise exception
    except httpx.TimeoutException as e:
        raise exceptions.InvokeTimeoutError from e
    except httpx.NetworkError as e:
        raise exceptions.NetworkError from e
    # TODO(MarshalX): add more exceptions


def _handle_response(response: httpx.Response) -> httpx.Response:
    if 200 <= response.status_code <= 299:
        return response

    error_response = Response(
        success=False,
        status_code=response.status_code,
        content=response.content,
        headers=_convert_headers_to_dict(response.headers),
    )
    error_content = load_json(response.content, strict=False)
    if error_content:
        error_response.content = t.cast(XrpcError, get_or_create(error_content, XrpcError, strict=False))

    if response.status_code in {401, 403}:
        raise exceptions.UnauthorizedError(error_response)
    if response.status_code == 400:
        raise exceptions.BadRequestError(error_response)
    if response.status_code in {409, 413, 502}:
        raise exceptions.NetworkError(error_response)

    raise exceptions.RequestException(error_response)


class RequestBase:
    _MANDATORY_HEADERS: t.ClassVar[t.Dict[str, str]] = {'User-Agent': _ATPROTO_SDK_USER_AGENT}

    def __init__(self) -> None:
        self._additional_headers: t.Dict[str, str] = {}
        self._additional_header_sources: t.List[t.Callable[[], t.Dict[str, str]]] = []

    def get_headers(self, additional_headers: t.Optional[t.Dict[str, str]] = None) -> t.Dict[str, str]:
        """Get headers for the request.

        Args:
            additional_headers: Additional headers.
                Overrides existing headers with the same name.

        Returns:
            Headers for the request.
        """
        # The order of calling `.update()` matters. It defines the priority of overriding.
        headers_lower = {
            **_normalize_headers(RequestBase._MANDATORY_HEADERS),
            **_normalize_headers(self._additional_headers),
        }

        for header_source in self._additional_header_sources:
            headers_lower.update(_normalize_headers(header_source()))

        if additional_headers:
            headers_lower.update(_normalize_headers(additional_headers))

        return _denormalize_headers(headers_lower)

    def set_additional_headers(self, headers: t.Dict[str, str]) -> None:
        """Set additional headers for the request.

        Args:
            headers: Additional headers.
        """
        self._additional_headers = headers.copy()

    def add_additional_headers_source(self, callback: t.Callable[[], t.Dict[str, str]]) -> None:
        """Add additional headers for the request.

        Args:
            callback: Function to get additional headers.
        """
        self._additional_header_sources.append(callback)

    def add_additional_header(self, header_name: str, header_value: str) -> None:
        """Add additional headers for the request.

        Note:
            This method overrides the existing header with the same name.

        Args:
            header_name: Header name.
            header_value: Header value.
        """
        self._additional_headers[header_name] = header_value

    def clone(self) -> te.Self:
        """Clone the client instance.

        Used to customize atproto proxy and set of labeler services.

        Returns:
            Cloned client instance.
        """
        cloned_request = type(self)()

        cloned_request._additional_headers = self._additional_headers.copy()
        cloned_request._additional_header_sources = self._additional_header_sources.copy()

        return cloned_request


class Request(RequestBase):
    """Class for handling requests errors and working with httpx.

    Args:
        **kwargs: Additional parameters for httpx.Client.
    """

    def __init__(self, **kwargs: t.Any) -> None:
        super().__init__()
        self._client = httpx.Client(follow_redirects=True, **kwargs)

    def _send_request(self, method: str, url: str, **kwargs: t.Any) -> httpx.Response:
        headers = self.get_headers(kwargs.pop('headers', None))

        try:
            response = self._client.request(method=method, url=url, headers=headers, **kwargs)
            return _handle_response(response)
        except Exception as e:
            _handle_request_errors(e)
            raise e

    def close(self) -> None:
        self._client.close()

    def get(self, *args: t.Any, **kwargs: t.Any) -> Response:
        return _parse_response(self._send_request('GET', *args, **kwargs))

    def post(self, *args: t.Any, **kwargs: t.Any) -> Response:
        return _parse_response(self._send_request('POST', *args, **kwargs))


class AsyncRequest(RequestBase):
    """Class for handling requests errors and working with httpx.

    Args:
        **kwargs: Additional parameters for httpx.AsyncClient.
    """

    def __init__(self, **kwargs: t.Any) -> None:
        super().__init__()
        self._client = httpx.AsyncClient(follow_redirects=True, **kwargs)

    async def _send_request(self, method: str, url: str, **kwargs: t.Any) -> httpx.Response:
        headers = self.get_headers(kwargs.pop('headers', None))

        try:
            response = await self._client.request(method=method, url=url, headers=headers, **kwargs)
            return _handle_response(response)
        except Exception as e:
            _handle_request_errors(e)
            raise e

    async def close(self) -> None:
        await self._client.aclose()

    async def get(self, *args: t.Any, **kwargs: t.Any) -> Response:
        return _parse_response(await self._send_request('GET', *args, **kwargs))

    async def post(self, *args: t.Any, **kwargs: t.Any) -> Response:
        return _parse_response(await self._send_request('POST', *args, **kwargs))



from atproto_client import models

RECORD_TYPE_TO_MODEL_CLASS = {
    'app.bsky.actor.profile': models.AppBskyActorProfile.Record,
    'app.bsky.feed.generator': models.AppBskyFeedGenerator.Record,
    'app.bsky.feed.like': models.AppBskyFeedLike.Record,
    'app.bsky.feed.post': models.AppBskyFeedPost.Record,
    'app.bsky.feed.postgate': models.AppBskyFeedPostgate.Record,
    'app.bsky.feed.repost': models.AppBskyFeedRepost.Record,
    'app.bsky.feed.threadgate': models.AppBskyFeedThreadgate.Record,
    'app.bsky.graph.block': models.AppBskyGraphBlock.Record,
    'app.bsky.graph.follow': models.AppBskyGraphFollow.Record,
    'app.bsky.graph.list': models.AppBskyGraphList.Record,
    'app.bsky.graph.listblock': models.AppBskyGraphListblock.Record,
    'app.bsky.graph.listitem': models.AppBskyGraphListitem.Record,
    'app.bsky.graph.starterpack': models.AppBskyGraphStarterpack.Record,
    'app.bsky.graph.verification': models.AppBskyGraphVerification.Record,
    'app.bsky.labeler.service': models.AppBskyLabelerService.Record,
    'chat.bsky.actor.declaration': models.ChatBskyActorDeclaration.Record,
    'com.atproto.lexicon.schema': models.ComAtprotoLexiconSchema.Record,
}



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from atproto_client.models import base, string_formats


class Data(base.DataModelBase):
    """Input data model for :obj:`com.atproto.admin.updateAccountSigningKey`."""

    did: string_formats.Did  #: Did.
    signing_key: string_formats.Did  #: Did-key formatted public key.


class DataDict(t.TypedDict):
    did: string_formats.Did  #: Did.
    signing_key: string_formats.Did  #: Did-key formatted public key.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`app.bsky.unspecced.getTaggedSuggestions`."""


class ParamsDict(t.TypedDict):
    pass


class Response(base.ResponseModelBase):
    """Output data model for :obj:`app.bsky.unspecced.getTaggedSuggestions`."""

    suggestions: t.List['models.AppBskyUnspeccedGetTaggedSuggestions.Suggestion']  #: Suggestions.


class Suggestion(base.ModelBase):
    """Definition model for :obj:`app.bsky.unspecced.getTaggedSuggestions`."""

    subject: string_formats.Uri  #: Subject.
    subject_type: t.Union[t.Literal['actor'], t.Literal['feed'], str]  #: Subject type.
    tag: str  #: Tag.

    py_type: t.Literal['app.bsky.unspecced.getTaggedSuggestions#suggestion'] = Field(
        default='app.bsky.unspecced.getTaggedSuggestions#suggestion', alias='$type', frozen=True
    )



import typing as t

import typing_extensions as te
from atproto_core.cid import CID
from pydantic import BaseModel, ConfigDict, Field


class IpldLink(BaseModel):
    """CID representation in JSON."""

    model_config = ConfigDict(extra='allow', populate_by_name=True, strict=True)

    link: str = Field(alias='$link')  #: CID.


class BlobRef(BaseModel):
    """Blob reference."""

    model_config = ConfigDict(extra='allow', populate_by_name=True, strict=True)

    mime_type: str = Field(alias='mimeType')  #: Mime type.
    size: int  #: Size in bytes.
    ref: t.Union[str, bytes, IpldLink]  #: CID.

    py_type: te.Literal['blob'] = Field(default='blob', alias='$type')

    @property
    def cid(self) -> 'CID':
        """Get CID."""
        if self.is_bytes_representation:
            return CID.decode(self.ref)

        return CID.decode(self.ref.link)

    @property
    def is_json_representation(self) -> bool:
        """Check if it is JSON representation.

        Returns:
            True if it is JSON representation.
        """
        return isinstance(self.ref, IpldLink)

    @property
    def is_bytes_representation(self) -> bool:
        """Check if it is bytes representation.

        Returns:
            True if it is bytes representation.
        """
        return isinstance(self.ref, (str, bytes))

    def to_json_representation(self) -> 'BlobRef':
        """Get JSON representation.

        Note:
            Used in XRPC, etc. where JSON is used.

        Warning:
            It returns new instance.

        Returns:
            BlobRef in JSON representation.
        """
        if self.is_json_representation:
            return BlobRef(mime_type=self.mime_type, size=self.size, ref=IpldLink(link=self.ref.link))

        return BlobRef(mime_type=self.mime_type, size=self.size, ref=IpldLink(link=self.ref))

    def to_bytes_representation(self) -> 'BlobRef':
        """Get bytes representation.

        Note:
            Used in Firehose, CAR, etc. where bytes are possible.

        Warning:
            It returns new instance.

        Returns:
            BlobRef in bytes representation.
        """
        if self.is_bytes_representation:
            return BlobRef(mime_type=self.mime_type, size=self.size, ref=self.ref)

        return BlobRef(mime_type=self.mime_type, size=self.size, ref=self.ref.link)



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

if t.TYPE_CHECKING:
    from atproto_client.models.unknown_type import UnknownInputType
from atproto_client.models import base


class Data(base.DataModelBase):
    """Input data model for :obj:`com.atproto.identity.submitPlcOperation`."""

    operation: 'UnknownInputType'  #: Operation.


class DataDict(t.TypedDict):
    operation: 'UnknownInputType'  #: Operation.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`app.bsky.graph.getStarterPack`."""

    starter_pack: string_formats.AtUri  #: Reference (AT-URI) of the starter pack record.


class ParamsDict(t.TypedDict):
    starter_pack: string_formats.AtUri  #: Reference (AT-URI) of the starter pack record.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`app.bsky.graph.getStarterPack`."""

    starter_pack: 'models.AppBskyGraphDefs.StarterPackView'  #: Starter pack.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client.models.unknown_type import UnknownType
from atproto_client.models import base


class IdentityInfo(base.ModelBase):
    """Definition model for :obj:`com.atproto.identity.defs`."""

    did: string_formats.Did  #: Did.
    did_doc: 'UnknownType'  #: The complete DID document for the identity.
    handle: string_formats.Handle  #: The validated handle of the account; or 'handle.invalid' if the handle did not bi-directionally match the DID document.

    py_type: t.Literal['com.atproto.identity.defs#identityInfo'] = Field(
        default='com.atproto.identity.defs#identityInfo', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
    from atproto_client.models.unknown_type import UnknownInputType
from atproto_client.models import base


class Data(base.DataModelBase):
    """Input data model for :obj:`tools.ozone.setting.upsertOption`."""

    key: string_formats.Nsid  #: Key.
    scope: t.Union[t.Literal['instance'], t.Literal['personal'], str]  #: Scope.
    value: 'UnknownInputType'  #: Value.
    description: t.Optional[str] = Field(default=None, max_length=2000)  #: Description.
    manager_role: t.Optional[
        t.Union[
            'models.ToolsOzoneTeamDefs.RoleModerator',
            'models.ToolsOzoneTeamDefs.RoleTriage',
            'models.ToolsOzoneTeamDefs.RoleAdmin',
            str,
        ]
    ] = None  #: Manager role.


class DataDict(t.TypedDict):
    key: string_formats.Nsid  #: Key.
    scope: t.Union[t.Literal['instance'], t.Literal['personal'], str]  #: Scope.
    value: 'UnknownInputType'  #: Value.
    description: te.NotRequired[t.Optional[str]]  #: Description.
    manager_role: te.NotRequired[
        t.Optional[
            t.Union[
                'models.ToolsOzoneTeamDefs.RoleModerator',
                'models.ToolsOzoneTeamDefs.RoleTriage',
                'models.ToolsOzoneTeamDefs.RoleAdmin',
                str,
            ]
        ]
    ]  #: Manager role.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`tools.ozone.setting.upsertOption`."""

    option: 'models.ToolsOzoneSettingDefs.Option'  #: Option.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class ProfileViewBasic(base.ModelBase):
    """Definition model for :obj:`chat.bsky.actor.defs`."""

    did: string_formats.Did  #: Did.
    handle: string_formats.Handle  #: Handle.
    associated: t.Optional['models.AppBskyActorDefs.ProfileAssociated'] = None  #: Associated.
    avatar: t.Optional[string_formats.Uri] = None  #: Avatar.
    chat_disabled: t.Optional[bool] = None  #: Set to true when the actor cannot actively participate in conversations.
    display_name: t.Optional[str] = Field(default=None, max_length=640)  #: Display name.
    labels: t.Optional[t.List['models.ComAtprotoLabelDefs.Label']] = None  #: Labels.
    verification: t.Optional['models.AppBskyActorDefs.VerificationState'] = None  #: Verification.
    viewer: t.Optional['models.AppBskyActorDefs.ViewerState'] = None  #: Viewer.

    py_type: t.Literal['chat.bsky.actor.defs#profileViewBasic'] = Field(
        default='chat.bsky.actor.defs#profileViewBasic', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from atproto_client.models import base


class Response(base.ResponseModelBase):
    """Output data model for :obj:`app.bsky.unspecced.getConfig`."""

    check_email_confirmed: t.Optional[bool] = None  #: Check email confirmed.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client.models.unknown_type import UnknownType
from atproto_client.models import base


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.server.refreshSession`."""

    access_jwt: str  #: Access jwt.
    did: string_formats.Did  #: Did.
    handle: string_formats.Handle  #: Handle.
    refresh_jwt: str  #: Refresh jwt.
    active: t.Optional[bool] = None  #: Active.
    did_doc: t.Optional['UnknownType'] = None  #: Did doc.
    status: t.Optional[t.Union[t.Literal['takendown'], t.Literal['suspended'], t.Literal['deactivated'], str]] = (
        None  #: Hosting status of the account. If not specified, then assume 'active'.
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`com.atproto.admin.getInviteCodes`."""

    cursor: t.Optional[str] = None  #: Cursor.
    limit: t.Optional[int] = Field(default=100, ge=1, le=500)  #: Limit.
    sort: t.Optional[t.Union[t.Literal['recent'], t.Literal['usage'], str]] = 'recent'  #: Sort.


class ParamsDict(t.TypedDict):
    cursor: te.NotRequired[t.Optional[str]]  #: Cursor.
    limit: te.NotRequired[t.Optional[int]]  #: Limit.
    sort: te.NotRequired[t.Optional[t.Union[t.Literal['recent'], t.Literal['usage'], str]]]  #: Sort.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.admin.getInviteCodes`."""

    codes: t.List['models.ComAtprotoServerDefs.InviteCode']  #: Codes.
    cursor: t.Optional[str] = None  #: Cursor.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te

from atproto_client.models import base, string_formats


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`com.atproto.server.getServiceAuth`."""

    aud: string_formats.Did  #: The DID of the service that the token will be used to authenticate with.
    exp: t.Optional[int] = (
        None  #: The time in Unix Epoch seconds that the JWT expires. Defaults to 60 seconds in the future. The service may enforce certain time bounds on tokens depending on the requested scope.
    )
    lxm: t.Optional[string_formats.Nsid] = None  #: Lexicon (XRPC) method to bind the requested token to.


class ParamsDict(t.TypedDict):
    aud: string_formats.Did  #: The DID of the service that the token will be used to authenticate with.
    exp: te.NotRequired[
        t.Optional[int]
    ]  #: The time in Unix Epoch seconds that the JWT expires. Defaults to 60 seconds in the future. The service may enforce certain time bounds on tokens depending on the requested scope.
    lxm: te.NotRequired[t.Optional[string_formats.Nsid]]  #: Lexicon (XRPC) method to bind the requested token to.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.server.getServiceAuth`."""

    token: str  #: Token.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te

from atproto_client.models import base, string_formats


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`tools.ozone.moderation.getRecord`."""

    uri: string_formats.AtUri  #: Uri.
    cid: t.Optional[string_formats.Cid] = None  #: Cid.


class ParamsDict(t.TypedDict):
    uri: string_formats.AtUri  #: Uri.
    cid: te.NotRequired[t.Optional[string_formats.Cid]]  #: Cid.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`tools.ozone.set.querySets`."""

    cursor: t.Optional[str] = None  #: Cursor.
    limit: t.Optional[int] = Field(default=50, ge=1, le=100)  #: Limit.
    name_prefix: t.Optional[str] = None  #: Name prefix.
    sort_by: t.Optional[t.Union[t.Literal['name'], t.Literal['createdAt'], t.Literal['updatedAt']]] = (
        'name'  #: Sort by.
    )
    sort_direction: t.Optional[t.Union[t.Literal['asc'], t.Literal['desc']]] = (
        'asc'  #: Defaults to ascending order of name field.
    )


class ParamsDict(t.TypedDict):
    cursor: te.NotRequired[t.Optional[str]]  #: Cursor.
    limit: te.NotRequired[t.Optional[int]]  #: Limit.
    name_prefix: te.NotRequired[t.Optional[str]]  #: Name prefix.
    sort_by: te.NotRequired[
        t.Optional[t.Union[t.Literal['name'], t.Literal['createdAt'], t.Literal['updatedAt']]]
    ]  #: Sort by.
    sort_direction: te.NotRequired[
        t.Optional[t.Union[t.Literal['asc'], t.Literal['desc']]]
    ]  #: Defaults to ascending order of name field.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`tools.ozone.set.querySets`."""

    sets: t.List['models.ToolsOzoneSetDefs.SetView']  #: Sets.
    cursor: t.Optional[str] = None  #: Cursor.



class AtProtocolError(Exception):
    """Base exception."""


class InvalidNsidError(AtProtocolError): ...


class InvalidAtUriError(AtProtocolError): ...


class InvalidCARFile(AtProtocolError): ...


class DAGCBORDecodingError(AtProtocolError): ...



import typing as t
from datetime import timedelta

from atproto_client.client.methods_mixin.time import TimeMethodsMixin
from atproto_client.client.session import (
    AsyncSessionChangeCallback,
    Session,
    SessionChangeCallback,
    SessionDispatcher,
    SessionEvent,
    SessionResponse,
    get_session_pds_endpoint,
)
from atproto_client.exceptions import LoginRequiredError


class SessionDispatchMixin:
    def on_session_change(self, callback: SessionChangeCallback) -> None:
        """Register a callback for session change event.

        Args:
            callback: A callback to be called when the session changes.
                The callback must accept two arguments: event and session.

        Note:
            Possible events: `SessionEvent.IMPORT`, `SessionEvent.CREATE`, `SessionEvent.REFRESH`.

        Tip:
            You should save the session string to persistent storage
            on `SessionEvent.CREATE` and `SessionEvent.REFRESH` event.

        Example:
            >>> from atproto import Client, SessionEvent, Session
            >>>
            >>> client = Client()
            >>>
            >>> @client.on_session_change
            >>> def on_session_change(event: SessionEvent, session: Session):
            >>>     print(event, session)
            >>>
            >>> # or you can use this syntax:
            >>> # client.on_session_change(on_session_change)

        Returns:
            :obj:`None`
        """
        self._session_dispatcher.on_session_change(callback)

    def _call_on_session_change_callbacks(self, event: SessionEvent) -> None:
        self._session_dispatcher.dispatch_session_change(event)


class AsyncSessionDispatchMixin:
    def on_session_change(self, callback: t.Union['AsyncSessionChangeCallback', 'SessionChangeCallback']) -> None:
        """Register a callback for session change event.

        Args:
            callback: A callback to be called when the session changes.
                The callback must accept two arguments: event and session.

        Note:
            Possible events: `SessionEvent.IMPORT`, `SessionEvent.CREATE`, `SessionEvent.REFRESH`.

        Note:
            You can register both synchronous and asynchronous callbacks.

        Tip:
            You should save the session string to persistent storage
            on `SessionEvent.CREATE` and `SessionEvent.REFRESH` event.

        Example:
            >>> from atproto import AsyncClient, SessionEvent, Session
            >>>
            >>> client = AsyncClient()
            >>>
            >>> @client.on_session_change
            >>> async def on_session_change(event: SessionEvent, session: Session):
            >>>     print(event, session)
            >>>
            >>> # or you can use this syntax:
            >>> # client.on_session_change(on_session_change)

        Returns:
            :obj:`None`
        """
        self._session_dispatcher.on_session_change(callback)

    async def _call_on_session_change_callbacks(self, event: SessionEvent) -> None:
        await self._session_dispatcher.dispatch_session_change_async(event)


class SessionMethodsMixin(TimeMethodsMixin):
    def __init__(self, *args: t.Any, **kwargs: t.Any) -> None:
        super().__init__(*args, **kwargs)
        self._session: t.Optional[Session] = None
        self._session_dispatcher = SessionDispatcher()

    def _register_auth_headers_source(self) -> None:
        self.request.add_additional_headers_source(self._get_access_auth_headers)

    def _should_refresh_session(self) -> bool:
        if not self._session or not self._session.access_jwt_payload or not self._session.access_jwt_payload.exp:
            raise LoginRequiredError

        expired_at = self.get_time_from_timestamp(self._session.access_jwt_payload.exp)
        expired_at = expired_at - timedelta(minutes=15)  # let's update the token a bit earlier than required

        return self.get_current_time() > expired_at

    def _set_or_update_session(self, session: SessionResponse, pds_endpoint: str) -> 'Session':
        if not self._session:
            self._session = Session(
                access_jwt=session.access_jwt,
                refresh_jwt=session.refresh_jwt,
                did=session.did,
                handle=session.handle,
                pds_endpoint=pds_endpoint,
            )
            self._session_dispatcher.set_session(self._session)
            self._register_auth_headers_source()
        else:
            self._session.access_jwt = session.access_jwt
            self._session.refresh_jwt = session.refresh_jwt
            self._session.did = session.did
            self._session.handle = session.handle
            self._session.pds_endpoint = pds_endpoint

        return self._session

    def _set_session_common(self, session: SessionResponse, current_pds: str) -> Session:
        pds_endpoint = get_session_pds_endpoint(session)
        if not pds_endpoint:
            # current_pds ends with xrpc endpoint, but this is not a problem
            # overhead is only 4-5 symbols in the exported session string
            pds_endpoint = current_pds

        self._update_pds_endpoint(pds_endpoint)
        return self._set_or_update_session(session, pds_endpoint)

    def _get_access_auth_headers(self) -> t.Dict[str, str]:
        if not self._session:
            return {}

        return {'Authorization': f'Bearer {self._session.access_jwt}'}

    def _get_refresh_auth_headers(self) -> t.Dict[str, str]:
        if not self._session:
            return {}

        return {'Authorization': f'Bearer {self._session.refresh_jwt}'}

    def _update_pds_endpoint(self, pds_endpoint: str) -> None:
        self.update_base_url(pds_endpoint)

    def export_session_string(self) -> str:
        """Export session string.

        Note:
            This method is useful for storing the session and reusing it later.

        Warning:
            You should use it if you create the client instance often.
            Because of server rate limits for `createSession`.
            Rate limited by handle.
            30/5 min, 300/day.

        Attention:
            You must export session at the end of the Client`s life cycle!
            Alternatively, you can subscribe to the session change event.
            Use :py:attr:`~on_session_change` to register handler.

        Example:
            >>> from atproto import Client
            >>> # the first time login with login and password
            >>> client = Client()
            >>> client.login('login', 'password')
            >>> session_string = client.export_session_string()
            >>> # store session_string somewhere.
            >>> # for example, in env and next time use it for login
            >>> client2 = Client()
            >>> client2.login(session_string=session_string)

        Returns:
            :obj:`str`: Session string.
        """
        if not self._session:
            raise LoginRequiredError

        return self._session.export()



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class SkeletonSearchPost(base.ModelBase):
    """Definition model for :obj:`app.bsky.unspecced.defs`."""

    uri: string_formats.AtUri  #: Uri.

    py_type: t.Literal['app.bsky.unspecced.defs#skeletonSearchPost'] = Field(
        default='app.bsky.unspecced.defs#skeletonSearchPost', alias='$type', frozen=True
    )


class SkeletonSearchActor(base.ModelBase):
    """Definition model for :obj:`app.bsky.unspecced.defs`."""

    did: string_formats.Did  #: Did.

    py_type: t.Literal['app.bsky.unspecced.defs#skeletonSearchActor'] = Field(
        default='app.bsky.unspecced.defs#skeletonSearchActor', alias='$type', frozen=True
    )


class SkeletonSearchStarterPack(base.ModelBase):
    """Definition model for :obj:`app.bsky.unspecced.defs`."""

    uri: string_formats.AtUri  #: Uri.

    py_type: t.Literal['app.bsky.unspecced.defs#skeletonSearchStarterPack'] = Field(
        default='app.bsky.unspecced.defs#skeletonSearchStarterPack', alias='$type', frozen=True
    )


class TrendingTopic(base.ModelBase):
    """Definition model for :obj:`app.bsky.unspecced.defs`."""

    link: str  #: Link.
    topic: str  #: Topic.
    description: t.Optional[str] = None  #: Description.
    display_name: t.Optional[str] = None  #: Display name.

    py_type: t.Literal['app.bsky.unspecced.defs#trendingTopic'] = Field(
        default='app.bsky.unspecced.defs#trendingTopic', alias='$type', frozen=True
    )


class SkeletonTrend(base.ModelBase):
    """Definition model for :obj:`app.bsky.unspecced.defs`."""

    dids: t.List[string_formats.Did]  #: Dids.
    display_name: str  #: Display name.
    link: str  #: Link.
    post_count: int  #: Post count.
    started_at: string_formats.DateTime  #: Started at.
    topic: str  #: Topic.
    category: t.Optional[str] = None  #: Category.
    status: t.Optional[t.Union[t.Literal['hot'], str]] = None  #: Status.

    py_type: t.Literal['app.bsky.unspecced.defs#skeletonTrend'] = Field(
        default='app.bsky.unspecced.defs#skeletonTrend', alias='$type', frozen=True
    )


class TrendView(base.ModelBase):
    """Definition model for :obj:`app.bsky.unspecced.defs`."""

    actors: t.List['models.AppBskyActorDefs.ProfileViewBasic']  #: Actors.
    display_name: str  #: Display name.
    link: str  #: Link.
    post_count: int  #: Post count.
    started_at: string_formats.DateTime  #: Started at.
    topic: str  #: Topic.
    category: t.Optional[str] = None  #: Category.
    status: t.Optional[t.Union[t.Literal['hot'], str]] = None  #: Status.

    py_type: t.Literal['app.bsky.unspecced.defs#trendView'] = Field(
        default='app.bsky.unspecced.defs#trendView', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class MessageRef(base.ModelBase):
    """Definition model for :obj:`chat.bsky.convo.defs`."""

    convo_id: str  #: Convo id.
    did: string_formats.Did  #: Did.
    message_id: str  #: Message id.

    py_type: t.Literal['chat.bsky.convo.defs#messageRef'] = Field(
        default='chat.bsky.convo.defs#messageRef', alias='$type', frozen=True
    )


class MessageInput(base.ModelBase):
    """Definition model for :obj:`chat.bsky.convo.defs`."""

    text: str = Field(max_length=10000)  #: Text.
    embed: t.Optional[
        te.Annotated[t.Union['models.AppBskyEmbedRecord.Main'], Field(default=None, discriminator='py_type')]
    ] = None  #: Embed.
    facets: t.Optional[t.List['models.AppBskyRichtextFacet.Main']] = (
        None  #: Annotations of text (mentions, URLs, hashtags, etc).
    )

    py_type: t.Literal['chat.bsky.convo.defs#messageInput'] = Field(
        default='chat.bsky.convo.defs#messageInput', alias='$type', frozen=True
    )


class MessageView(base.ModelBase):
    """Definition model for :obj:`chat.bsky.convo.defs`."""

    id: str  #: Id.
    rev: str  #: Rev.
    sender: 'models.ChatBskyConvoDefs.MessageViewSender'  #: Sender.
    sent_at: string_formats.DateTime  #: Sent at.
    text: str = Field(max_length=10000)  #: Text.
    embed: t.Optional[
        te.Annotated[t.Union['models.AppBskyEmbedRecord.View'], Field(default=None, discriminator='py_type')]
    ] = None  #: Embed.
    facets: t.Optional[t.List['models.AppBskyRichtextFacet.Main']] = (
        None  #: Annotations of text (mentions, URLs, hashtags, etc).
    )
    reactions: t.Optional[t.List['models.ChatBskyConvoDefs.ReactionView']] = (
        None  #: Reactions to this message, in ascending order of creation time.
    )

    py_type: t.Literal['chat.bsky.convo.defs#messageView'] = Field(
        default='chat.bsky.convo.defs#messageView', alias='$type', frozen=True
    )


class DeletedMessageView(base.ModelBase):
    """Definition model for :obj:`chat.bsky.convo.defs`."""

    id: str  #: Id.
    rev: str  #: Rev.
    sender: 'models.ChatBskyConvoDefs.MessageViewSender'  #: Sender.
    sent_at: string_formats.DateTime  #: Sent at.

    py_type: t.Literal['chat.bsky.convo.defs#deletedMessageView'] = Field(
        default='chat.bsky.convo.defs#deletedMessageView', alias='$type', frozen=True
    )


class MessageViewSender(base.ModelBase):
    """Definition model for :obj:`chat.bsky.convo.defs`."""

    did: string_formats.Did  #: Did.

    py_type: t.Literal['chat.bsky.convo.defs#messageViewSender'] = Field(
        default='chat.bsky.convo.defs#messageViewSender', alias='$type', frozen=True
    )


class ReactionView(base.ModelBase):
    """Definition model for :obj:`chat.bsky.convo.defs`."""

    created_at: string_formats.DateTime  #: Created at.
    sender: 'models.ChatBskyConvoDefs.ReactionViewSender'  #: Sender.
    value: str  #: Value.

    py_type: t.Literal['chat.bsky.convo.defs#reactionView'] = Field(
        default='chat.bsky.convo.defs#reactionView', alias='$type', frozen=True
    )


class ReactionViewSender(base.ModelBase):
    """Definition model for :obj:`chat.bsky.convo.defs`."""

    did: string_formats.Did  #: Did.

    py_type: t.Literal['chat.bsky.convo.defs#reactionViewSender'] = Field(
        default='chat.bsky.convo.defs#reactionViewSender', alias='$type', frozen=True
    )


class MessageAndReactionView(base.ModelBase):
    """Definition model for :obj:`chat.bsky.convo.defs`."""

    message: 'models.ChatBskyConvoDefs.MessageView'  #: Message.
    reaction: 'models.ChatBskyConvoDefs.ReactionView'  #: Reaction.

    py_type: t.Literal['chat.bsky.convo.defs#messageAndReactionView'] = Field(
        default='chat.bsky.convo.defs#messageAndReactionView', alias='$type', frozen=True
    )


class ConvoView(base.ModelBase):
    """Definition model for :obj:`chat.bsky.convo.defs`."""

    id: str  #: Id.
    members: t.List['models.ChatBskyActorDefs.ProfileViewBasic']  #: Members.
    muted: bool  #: Muted.
    rev: str  #: Rev.
    unread_count: int  #: Unread count.
    last_message: t.Optional[
        te.Annotated[
            t.Union['models.ChatBskyConvoDefs.MessageView', 'models.ChatBskyConvoDefs.DeletedMessageView'],
            Field(default=None, discriminator='py_type'),
        ]
    ] = None  #: Last message.
    last_reaction: t.Optional[
        te.Annotated[
            t.Union['models.ChatBskyConvoDefs.MessageAndReactionView'], Field(default=None, discriminator='py_type')
        ]
    ] = None  #: Last reaction.
    status: t.Optional[t.Union[t.Literal['request'], t.Literal['accepted'], str]] = None  #: Status.

    py_type: t.Literal['chat.bsky.convo.defs#convoView'] = Field(
        default='chat.bsky.convo.defs#convoView', alias='$type', frozen=True
    )


class LogBeginConvo(base.ModelBase):
    """Definition model for :obj:`chat.bsky.convo.defs`."""

    convo_id: str  #: Convo id.
    rev: str  #: Rev.

    py_type: t.Literal['chat.bsky.convo.defs#logBeginConvo'] = Field(
        default='chat.bsky.convo.defs#logBeginConvo', alias='$type', frozen=True
    )


class LogAcceptConvo(base.ModelBase):
    """Definition model for :obj:`chat.bsky.convo.defs`."""

    convo_id: str  #: Convo id.
    rev: str  #: Rev.

    py_type: t.Literal['chat.bsky.convo.defs#logAcceptConvo'] = Field(
        default='chat.bsky.convo.defs#logAcceptConvo', alias='$type', frozen=True
    )


class LogLeaveConvo(base.ModelBase):
    """Definition model for :obj:`chat.bsky.convo.defs`."""

    convo_id: str  #: Convo id.
    rev: str  #: Rev.

    py_type: t.Literal['chat.bsky.convo.defs#logLeaveConvo'] = Field(
        default='chat.bsky.convo.defs#logLeaveConvo', alias='$type', frozen=True
    )


class LogMuteConvo(base.ModelBase):
    """Definition model for :obj:`chat.bsky.convo.defs`."""

    convo_id: str  #: Convo id.
    rev: str  #: Rev.

    py_type: t.Literal['chat.bsky.convo.defs#logMuteConvo'] = Field(
        default='chat.bsky.convo.defs#logMuteConvo', alias='$type', frozen=True
    )


class LogUnmuteConvo(base.ModelBase):
    """Definition model for :obj:`chat.bsky.convo.defs`."""

    convo_id: str  #: Convo id.
    rev: str  #: Rev.

    py_type: t.Literal['chat.bsky.convo.defs#logUnmuteConvo'] = Field(
        default='chat.bsky.convo.defs#logUnmuteConvo', alias='$type', frozen=True
    )


class LogCreateMessage(base.ModelBase):
    """Definition model for :obj:`chat.bsky.convo.defs`."""

    convo_id: str  #: Convo id.
    message: te.Annotated[
        t.Union['models.ChatBskyConvoDefs.MessageView', 'models.ChatBskyConvoDefs.DeletedMessageView'],
        Field(discriminator='py_type'),
    ]  #: Message.
    rev: str  #: Rev.

    py_type: t.Literal['chat.bsky.convo.defs#logCreateMessage'] = Field(
        default='chat.bsky.convo.defs#logCreateMessage', alias='$type', frozen=True
    )


class LogDeleteMessage(base.ModelBase):
    """Definition model for :obj:`chat.bsky.convo.defs`."""

    convo_id: str  #: Convo id.
    message: te.Annotated[
        t.Union['models.ChatBskyConvoDefs.MessageView', 'models.ChatBskyConvoDefs.DeletedMessageView'],
        Field(discriminator='py_type'),
    ]  #: Message.
    rev: str  #: Rev.

    py_type: t.Literal['chat.bsky.convo.defs#logDeleteMessage'] = Field(
        default='chat.bsky.convo.defs#logDeleteMessage', alias='$type', frozen=True
    )


class LogReadMessage(base.ModelBase):
    """Definition model for :obj:`chat.bsky.convo.defs`."""

    convo_id: str  #: Convo id.
    message: te.Annotated[
        t.Union['models.ChatBskyConvoDefs.MessageView', 'models.ChatBskyConvoDefs.DeletedMessageView'],
        Field(discriminator='py_type'),
    ]  #: Message.
    rev: str  #: Rev.

    py_type: t.Literal['chat.bsky.convo.defs#logReadMessage'] = Field(
        default='chat.bsky.convo.defs#logReadMessage', alias='$type', frozen=True
    )


class LogAddReaction(base.ModelBase):
    """Definition model for :obj:`chat.bsky.convo.defs`."""

    convo_id: str  #: Convo id.
    message: te.Annotated[
        t.Union['models.ChatBskyConvoDefs.MessageView', 'models.ChatBskyConvoDefs.DeletedMessageView'],
        Field(discriminator='py_type'),
    ]  #: Message.
    reaction: 'models.ChatBskyConvoDefs.ReactionView'  #: Reaction.
    rev: str  #: Rev.

    py_type: t.Literal['chat.bsky.convo.defs#logAddReaction'] = Field(
        default='chat.bsky.convo.defs#logAddReaction', alias='$type', frozen=True
    )


class LogRemoveReaction(base.ModelBase):
    """Definition model for :obj:`chat.bsky.convo.defs`."""

    convo_id: str  #: Convo id.
    message: te.Annotated[
        t.Union['models.ChatBskyConvoDefs.MessageView', 'models.ChatBskyConvoDefs.DeletedMessageView'],
        Field(discriminator='py_type'),
    ]  #: Message.
    reaction: 'models.ChatBskyConvoDefs.ReactionView'  #: Reaction.
    rev: str  #: Rev.

    py_type: t.Literal['chat.bsky.convo.defs#logRemoveReaction'] = Field(
        default='chat.bsky.convo.defs#logRemoveReaction', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

from atproto_client.models import base, string_formats


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`app.bsky.unspecced.getSuggestedStarterPacksSkeleton`."""

    limit: t.Optional[int] = Field(default=10, ge=1, le=25)  #: Limit.
    viewer: t.Optional[string_formats.Did] = (
        None  #: DID of the account making the request (not included for public/unauthenticated queries).
    )


class ParamsDict(t.TypedDict):
    limit: te.NotRequired[t.Optional[int]]  #: Limit.
    viewer: te.NotRequired[
        t.Optional[string_formats.Did]
    ]  #: DID of the account making the request (not included for public/unauthenticated queries).


class Response(base.ResponseModelBase):
    """Output data model for :obj:`app.bsky.unspecced.getSuggestedStarterPacksSkeleton`."""

    starter_packs: t.List[string_formats.AtUri]  #: Starter packs.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from pydantic import Field

from atproto_client.models import base


class RecordDeleted(base.ModelBase):
    """Definition model for :obj:`app.bsky.notification.defs`."""

    py_type: t.Literal['app.bsky.notification.defs#recordDeleted'] = Field(
        default='app.bsky.notification.defs#recordDeleted', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
    from atproto_client.models.unknown_type import UnknownType
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`com.atproto.repo.listRecords`."""

    collection: string_formats.Nsid  #: The NSID of the record type.
    repo: string_formats.AtIdentifier  #: The handle or DID of the repo.
    cursor: t.Optional[str] = None  #: Cursor.
    limit: t.Optional[int] = Field(default=50, ge=1, le=100)  #: The number of records to return.
    reverse: t.Optional[bool] = None  #: Flag to reverse the order of the returned records.


class ParamsDict(t.TypedDict):
    collection: string_formats.Nsid  #: The NSID of the record type.
    repo: string_formats.AtIdentifier  #: The handle or DID of the repo.
    cursor: te.NotRequired[t.Optional[str]]  #: Cursor.
    limit: te.NotRequired[t.Optional[int]]  #: The number of records to return.
    reverse: te.NotRequired[t.Optional[bool]]  #: Flag to reverse the order of the returned records.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.repo.listRecords`."""

    records: t.List['models.ComAtprotoRepoListRecords.Record']  #: Records.
    cursor: t.Optional[str] = None  #: Cursor.


class Record(base.ModelBase):
    """Definition model for :obj:`com.atproto.repo.listRecords`."""

    cid: string_formats.Cid  #: Cid.
    uri: string_formats.AtUri  #: Uri.
    value: 'UnknownType'  #: Value.

    py_type: t.Literal['com.atproto.repo.listRecords#record'] = Field(
        default='com.atproto.repo.listRecords#record', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`app.bsky.unspecced.getSuggestedUsers`."""

    category: t.Optional[str] = None  #: Category of users to get suggestions for.
    limit: t.Optional[int] = Field(default=25, ge=1, le=50)  #: Limit.


class ParamsDict(t.TypedDict):
    category: te.NotRequired[t.Optional[str]]  #: Category of users to get suggestions for.
    limit: te.NotRequired[t.Optional[int]]  #: Limit.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`app.bsky.unspecced.getSuggestedUsers`."""

    actors: t.List['models.AppBskyActorDefs.ProfileView']  #: Actors.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`tools.ozone.moderation.queryEvents`."""

    added_labels: t.Optional[t.List[str]] = (
        None  #: If specified, only events where all of these labels were added are returned.
    )
    added_tags: t.Optional[t.List[str]] = (
        None  #: If specified, only events where all of these tags were added are returned.
    )
    collections: t.Optional[t.List[string_formats.Nsid]] = Field(
        default=None, max_length=20
    )  #: If specified, only events where the subject belongs to the given collections will be returned. When subjectType is set to 'account', this will be ignored.
    comment: t.Optional[str] = (
        None  #: If specified, only events with comments containing the keyword are returned. Apply || separator to use multiple keywords and match using OR condition.
    )
    created_after: t.Optional[string_formats.DateTime] = None  #: Retrieve events created after a given timestamp.
    created_before: t.Optional[string_formats.DateTime] = None  #: Retrieve events created before a given timestamp.
    created_by: t.Optional[string_formats.Did] = None  #: Created by.
    cursor: t.Optional[str] = None  #: Cursor.
    has_comment: t.Optional[bool] = None  #: If true, only events with comments are returned.
    include_all_user_records: t.Optional[bool] = (
        False  #: If true, events on all record types (posts, lists, profile etc.) or records from given 'collections' param, owned by the did are returned.
    )
    limit: t.Optional[int] = Field(default=50, ge=1, le=100)  #: Limit.
    policies: t.Optional[t.List[str]] = (
        None  #: Policies. If specified, only events where the action policies match any of the given policies are returned.
    )
    removed_labels: t.Optional[t.List[str]] = (
        None  #: If specified, only events where all of these labels were removed are returned.
    )
    removed_tags: t.Optional[t.List[str]] = (
        None  #: If specified, only events where all of these tags were removed are returned.
    )
    report_types: t.Optional[t.List[str]] = None  #: Report types.
    sort_direction: t.Optional[t.Union[t.Literal['asc'], t.Literal['desc']]] = (
        'desc'  #: Sort direction for the events. Defaults to descending order of created at timestamp.
    )
    subject: t.Optional[string_formats.Uri] = None  #: Subject.
    subject_type: t.Optional[t.Union[t.Literal['account'], t.Literal['record'], str]] = (
        None  #: If specified, only events where the subject is of the given type (account or record) will be returned. When this is set to 'account' the 'collections' parameter will be ignored. When includeAllUserRecords or subject is set, this will be ignored.
    )
    types: t.Optional[t.List[str]] = (
        None  #: The types of events (fully qualified string in the format of tools.ozone.moderation.defs#modEvent<name>) to filter by. If not specified, all events are returned.
    )


class ParamsDict(t.TypedDict):
    added_labels: te.NotRequired[
        t.Optional[t.List[str]]
    ]  #: If specified, only events where all of these labels were added are returned.
    added_tags: te.NotRequired[
        t.Optional[t.List[str]]
    ]  #: If specified, only events where all of these tags were added are returned.
    collections: te.NotRequired[
        t.Optional[t.List[string_formats.Nsid]]
    ]  #: If specified, only events where the subject belongs to the given collections will be returned. When subjectType is set to 'account', this will be ignored.
    comment: te.NotRequired[
        t.Optional[str]
    ]  #: If specified, only events with comments containing the keyword are returned. Apply || separator to use multiple keywords and match using OR condition.
    created_after: te.NotRequired[
        t.Optional[string_formats.DateTime]
    ]  #: Retrieve events created after a given timestamp.
    created_before: te.NotRequired[
        t.Optional[string_formats.DateTime]
    ]  #: Retrieve events created before a given timestamp.
    created_by: te.NotRequired[t.Optional[string_formats.Did]]  #: Created by.
    cursor: te.NotRequired[t.Optional[str]]  #: Cursor.
    has_comment: te.NotRequired[t.Optional[bool]]  #: If true, only events with comments are returned.
    include_all_user_records: te.NotRequired[
        t.Optional[bool]
    ]  #: If true, events on all record types (posts, lists, profile etc.) or records from given 'collections' param, owned by the did are returned.
    limit: te.NotRequired[t.Optional[int]]  #: Limit.
    policies: te.NotRequired[
        t.Optional[t.List[str]]
    ]  #: Policies. If specified, only events where the action policies match any of the given policies are returned.
    removed_labels: te.NotRequired[
        t.Optional[t.List[str]]
    ]  #: If specified, only events where all of these labels were removed are returned.
    removed_tags: te.NotRequired[
        t.Optional[t.List[str]]
    ]  #: If specified, only events where all of these tags were removed are returned.
    report_types: te.NotRequired[t.Optional[t.List[str]]]  #: Report types.
    sort_direction: te.NotRequired[
        t.Optional[t.Union[t.Literal['asc'], t.Literal['desc']]]
    ]  #: Sort direction for the events. Defaults to descending order of created at timestamp.
    subject: te.NotRequired[t.Optional[string_formats.Uri]]  #: Subject.
    subject_type: te.NotRequired[
        t.Optional[t.Union[t.Literal['account'], t.Literal['record'], str]]
    ]  #: If specified, only events where the subject is of the given type (account or record) will be returned. When this is set to 'account' the 'collections' parameter will be ignored. When includeAllUserRecords or subject is set, this will be ignored.
    types: te.NotRequired[
        t.Optional[t.List[str]]
    ]  #: The types of events (fully qualified string in the format of tools.ozone.moderation.defs#modEvent<name>) to filter by. If not specified, all events are returned.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`tools.ozone.moderation.queryEvents`."""

    events: t.List['models.ToolsOzoneModerationDefs.ModEventView']  #: Events.
    cursor: t.Optional[str] = None  #: Cursor.



import binascii
import typing as t
from datetime import datetime, timezone

from atproto_crypto.verify import verify_signature
from pydantic import BaseModel, ConfigDict
from pydantic_core import from_json

from atproto_server.auth.utils import base64url_decode
from atproto_server.exceptions import (
    TokenDecodeError,
    TokenExpiredSignatureError,
    TokenImmatureSignatureError,
    TokenInvalidAudienceError,
    TokenInvalidIssuedAtError,
    TokenInvalidSignatureError,
)

GetSigningKeyCallback = t.Callable[[str, bool], str]
GetSigningKeyCallbackAsync = t.Callable[[str, bool], t.Coroutine[t.Any, t.Any, str]]


class JwtPayload(BaseModel):
    """The payload of the JWT.

    Based on https://www.rfc-editor.org/rfc/rfc7519#section-4.1
    """

    model_config = ConfigDict(extra='allow', strict=True)

    iss: t.Optional[str] = None  #: Issuer (DID).
    sub: t.Optional[str] = None  #: Subject (DID).
    aud: t.Optional[t.Union[str, t.List[str]]] = None  #: Audience (DID).
    jti: t.Optional[str] = None  #: JWT ID. Presented in Refresh Token.
    nbf: t.Optional[int] = None  #: Not Before. Not used by ATProto.
    exp: t.Optional[int] = None  #: Expiration Time.
    iat: t.Optional[int] = None  #: Issued At.
    scope: t.Optional[str] = None  #: Scope. ATProto specific.
    # ... any other JWT Claim Set member.


def parse_jwt(jwt: t.Union[str, bytes]) -> t.Tuple[bytes, bytes, t.Dict[str, t.Any], bytes]:
    """Parse the given JWT.

    Args:
        jwt: The JWT to parse.

    Returns:
        :obj:`tuple` of :obj:`bytes`, :obj:`bytes`, :obj:`dict`, :obj:`bytes`:
        The parsed JWT: payload, signing input, header, signature.
    """
    if isinstance(jwt, str):
        jwt = jwt.encode('UTF-8')

    try:
        signing_input, crypto_segment = jwt.rsplit(b'.', 1)
        header_segment, payload_segment = signing_input.split(b'.', 1)
    except ValueError as e:
        raise TokenDecodeError('Not enough segments') from e

    try:
        header_data = base64url_decode(header_segment)
    except (TypeError, binascii.Error) as e:
        raise TokenDecodeError('Invalid header padding') from e

    try:
        header = from_json(header_data)
    except ValueError as e:
        raise TokenDecodeError(f'Invalid header string: {e}') from e

    if not isinstance(header, dict):
        raise TokenDecodeError('Invalid header string: must be a json object')

    header = t.cast(t.Dict[str, t.Any], from_json(header_data))  # we expect an object in header

    try:
        payload = base64url_decode(payload_segment)
    except (TypeError, binascii.Error) as e:
        raise TokenDecodeError('Invalid payload padding') from e

    try:
        signature = base64url_decode(crypto_segment)
    except (TypeError, binascii.Error) as e:
        raise TokenDecodeError('Invalid crypto padding') from e

    return payload, signing_input, header, signature


def decode_jwt_payload(payload: t.Union[str, bytes]) -> JwtPayload:
    """Decode the given JWT payload.

    Args:
        payload: The JWT payload to decode.

    Returns:
        :obj:`JwtPayload`: The decoded payload of the given JWT.
    """
    try:
        plain_payload = from_json(payload)
    except ValueError as e:
        raise TokenDecodeError(f'Invalid payload string: {e}') from e
    if not isinstance(plain_payload, dict):
        raise TokenDecodeError('Invalid payload string: must be a json object')

    try:
        return JwtPayload(**plain_payload)  # type: ignore  # noqa: PGH003
    except Exception as e:
        raise TokenDecodeError(f'Invalid payload string: {e}') from e


def get_jwt_payload(jwt: str) -> JwtPayload:
    """Return the payload of the given JWT.

    Args:
        jwt: The JWT to get the payload from.

    Returns:
        :obj:`JwtPayload`: The payload of the given JWT.
    """
    payload, *_ = parse_jwt(jwt)
    return decode_jwt_payload(payload)


def _validate_exp(
    exp: int,
    now: float,
    leeway: float,
) -> None:
    try:
        exp = int(exp)
    except ValueError as e:
        raise TokenDecodeError('Expiration Time claim (exp) must be an integer.') from e

    if exp <= (now - leeway):
        raise TokenExpiredSignatureError('Signature has expired')


def _validate_iat(
    iat: int,
    now: float,
    leeway: float,
) -> None:
    try:
        iat = int(iat)
    except ValueError as e:
        raise TokenInvalidIssuedAtError('Issued At claim (iat) must be an integer.') from e
    if iat > (now + leeway):
        raise TokenImmatureSignatureError('The token is not yet valid (iat)')


def validate_jwt_payload(payload: JwtPayload, leeway: int = 0) -> None:
    """Validate the given JWT payload.

    Args:
        payload: The JWT payload to validate.
        leeway: The leeway in seconds to accept when verifying time claims (exp, iat).

    Returns:
        :obj:`None`: The payload is valid.

    Raises:
        TokenDecodeError: If the given JWT is invalid.
        TokenExpiredSignatureError: If the given JWT is expired.
        TokenImmatureSignatureError: If the given JWT is immature.
        TokenInvalidIssuedAtError: If the given JWT has invalid issued at.
    """
    now = datetime.now(tz=timezone.utc).timestamp()

    if payload.exp is not None:
        _validate_exp(payload.exp, now, leeway)
    if payload.iat is not None:
        _validate_iat(payload.iat, now, leeway)


def _verify_signature(signing_key: str, signing_input: bytes, signature: bytes) -> bool:
    try:
        return verify_signature(signing_key, signing_input, signature)
    except Exception as e:
        raise TokenInvalidSignatureError('Could not verify JWT signature') from e


def verify_jwt(
    jwt: str, get_signing_key_callback: GetSigningKeyCallback, own_did: t.Optional[str] = None
) -> JwtPayload:
    """Verify the given JWT.

    Args:
        jwt: The JWT to verify.
        get_signing_key_callback: The callback to get the signing key.
        own_did: The DID of the service (aud).

    Returns:
        :obj:`JwtPayload`: The payload of the given JWT.

    Raises:
        TokenDecodeError: If the given JWT is invalid.
        TokenExpiredSignatureError: If the given JWT is expired.
        TokenImmatureSignatureError: If the given JWT is immature.
        TokenInvalidAudienceError: If the given JWT has invalid audience.
        TokenInvalidIssuedAtError: If the given JWT has invalid issued at.
        TokenInvalidSignatureError: If the given JWT has invalid signature.
    """
    plain_payload, signing_input, _, signature = parse_jwt(jwt)

    payload = decode_jwt_payload(plain_payload)
    validate_jwt_payload(payload)

    if own_did and payload.aud != own_did:
        raise TokenInvalidAudienceError('Invalid subject')

    if payload.iss is None:
        raise TokenDecodeError('Invalid payload. Expected not None iss')

    signing_key = get_signing_key_callback(payload.iss, False)
    if _verify_signature(signing_key, signing_input, signature):
        return payload

    fresh_signing_key = get_signing_key_callback(payload.iss, True)  # get signing key without a cache
    if fresh_signing_key == signing_key:
        raise TokenInvalidSignatureError('Invalid signature even with fresh signing key it is equal to the old one)')

    if _verify_signature(fresh_signing_key, signing_input, signature):
        return payload

    # this code should be unreachable
    # verifying methods must raise exception before
    raise TokenInvalidSignatureError('Invalid signature')


async def verify_jwt_async(
    jwt: str, get_signing_key_callback: GetSigningKeyCallbackAsync, own_did: t.Optional[str] = None
) -> JwtPayload:
    """Asynchronously verifies the given JWT.

    Args:
        jwt: The JWT to verify.
        get_signing_key_callback: The callback to get the signing key.
        own_did: The DID of the service (aud).

    Returns:
        :obj:`JwtPayload`: The payload of the given JWT.

    Raises:
        TokenDecodeError: If the given JWT is invalid.
        TokenExpiredSignatureError: If the given JWT is expired.
        TokenImmatureSignatureError: If the given JWT is immature.
        TokenInvalidAudienceError: If the given JWT has invalid audience.
        TokenInvalidIssuedAtError: If the given JWT has invalid issued at.
        TokenInvalidSignatureError: If the given JWT has invalid signature.
    """
    plain_payload, signing_input, _, signature = parse_jwt(jwt)

    payload = decode_jwt_payload(plain_payload)
    validate_jwt_payload(payload)

    if own_did and payload.aud != own_did:
        raise TokenInvalidAudienceError('Invalid subject')

    if payload.iss is None:
        raise TokenDecodeError('Invalid payload. Expected not None iss')

    signing_key = await get_signing_key_callback(payload.iss, False)
    if _verify_signature(signing_key, signing_input, signature):
        return payload

    fresh_signing_key = await get_signing_key_callback(payload.iss, True)  # get signing key without a cache
    if fresh_signing_key == signing_key:
        raise TokenInvalidSignatureError('Invalid signature even with fresh signing key it is equal to the old one)')

    if _verify_signature(fresh_signing_key, signing_input, signature):
        return payload

    # this code should be unreachable
    # verifying methods must raise exception before
    raise TokenInvalidSignatureError('Invalid signature')



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
    from atproto_client.models.unknown_type import UnknownInputType
from atproto_client.models import base


class Data(base.DataModelBase):
    """Input data model for :obj:`com.atproto.repo.putRecord`."""

    collection: string_formats.Nsid  #: The NSID of the record collection.
    record: 'UnknownInputType'  #: The record to write.
    repo: string_formats.AtIdentifier  #: The handle or DID of the repo (aka, current account).
    rkey: string_formats.RecordKey = Field(max_length=512)  #: The Record Key.
    swap_commit: t.Optional[string_formats.Cid] = None  #: Compare and swap with the previous commit by CID.
    swap_record: t.Optional[string_formats.Cid] = (
        None  #: Compare and swap with the previous record by CID. WARNING: nullable and optional field; may cause problems with golang implementation.
    )
    validate_: t.Optional[bool] = (
        None  #: Can be set to 'false' to skip Lexicon schema validation of record data, 'true' to require it, or leave unset to validate only for known Lexicons.
    )


class DataDict(t.TypedDict):
    collection: string_formats.Nsid  #: The NSID of the record collection.
    record: 'UnknownInputType'  #: The record to write.
    repo: string_formats.AtIdentifier  #: The handle or DID of the repo (aka, current account).
    rkey: string_formats.RecordKey  #: The Record Key.
    swap_commit: te.NotRequired[t.Optional[string_formats.Cid]]  #: Compare and swap with the previous commit by CID.
    swap_record: te.NotRequired[
        t.Optional[string_formats.Cid]
    ]  #: Compare and swap with the previous record by CID. WARNING: nullable and optional field; may cause problems with golang implementation.
    validate: te.NotRequired[
        t.Optional[bool]
    ]  #: Can be set to 'false' to skip Lexicon schema validation of record data, 'true' to require it, or leave unset to validate only for known Lexicons.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.repo.putRecord`."""

    cid: string_formats.Cid  #: Cid.
    uri: string_formats.AtUri  #: Uri.
    commit: t.Optional['models.ComAtprotoRepoDefs.CommitMeta'] = None  #: Commit.
    validation_status: t.Optional[t.Union[t.Literal['valid'], t.Literal['unknown'], str]] = None  #: Validation status.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`chat.bsky.convo.getLog`."""

    cursor: t.Optional[str] = None  #: Cursor.


class ParamsDict(t.TypedDict):
    cursor: te.NotRequired[t.Optional[str]]  #: Cursor.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`chat.bsky.convo.getLog`."""

    logs: t.List[
        te.Annotated[
            t.Union[
                'models.ChatBskyConvoDefs.LogBeginConvo',
                'models.ChatBskyConvoDefs.LogAcceptConvo',
                'models.ChatBskyConvoDefs.LogLeaveConvo',
                'models.ChatBskyConvoDefs.LogMuteConvo',
                'models.ChatBskyConvoDefs.LogUnmuteConvo',
                'models.ChatBskyConvoDefs.LogCreateMessage',
                'models.ChatBskyConvoDefs.LogDeleteMessage',
                'models.ChatBskyConvoDefs.LogReadMessage',
                'models.ChatBskyConvoDefs.LogAddReaction',
                'models.ChatBskyConvoDefs.LogRemoveReaction',
            ],
            Field(discriminator='py_type'),
        ]
    ]  #: Logs.
    cursor: t.Optional[str] = None  #: Cursor.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
    from atproto_client.models.unknown_type import UnknownType
from atproto_client.models import base


class ModEventView(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`."""

    created_at: string_formats.DateTime  #: Created at.
    created_by: string_formats.Did  #: Created by.
    event: te.Annotated[
        t.Union[
            'models.ToolsOzoneModerationDefs.ModEventTakedown',
            'models.ToolsOzoneModerationDefs.ModEventReverseTakedown',
            'models.ToolsOzoneModerationDefs.ModEventComment',
            'models.ToolsOzoneModerationDefs.ModEventReport',
            'models.ToolsOzoneModerationDefs.ModEventLabel',
            'models.ToolsOzoneModerationDefs.ModEventAcknowledge',
            'models.ToolsOzoneModerationDefs.ModEventEscalate',
            'models.ToolsOzoneModerationDefs.ModEventMute',
            'models.ToolsOzoneModerationDefs.ModEventUnmute',
            'models.ToolsOzoneModerationDefs.ModEventMuteReporter',
            'models.ToolsOzoneModerationDefs.ModEventUnmuteReporter',
            'models.ToolsOzoneModerationDefs.ModEventEmail',
            'models.ToolsOzoneModerationDefs.ModEventResolveAppeal',
            'models.ToolsOzoneModerationDefs.ModEventDivert',
            'models.ToolsOzoneModerationDefs.ModEventTag',
            'models.ToolsOzoneModerationDefs.AccountEvent',
            'models.ToolsOzoneModerationDefs.IdentityEvent',
            'models.ToolsOzoneModerationDefs.RecordEvent',
            'models.ToolsOzoneModerationDefs.ModEventPriorityScore',
        ],
        Field(discriminator='py_type'),
    ]  #: Event.
    id: int  #: Id.
    subject: te.Annotated[
        t.Union[
            'models.ComAtprotoAdminDefs.RepoRef',
            'models.ComAtprotoRepoStrongRef.Main',
            'models.ChatBskyConvoDefs.MessageRef',
        ],
        Field(discriminator='py_type'),
    ]  #: Subject.
    subject_blob_cids: t.List[str]  #: Subject blob cids.
    creator_handle: t.Optional[str] = None  #: Creator handle.
    subject_handle: t.Optional[str] = None  #: Subject handle.

    py_type: t.Literal['tools.ozone.moderation.defs#modEventView'] = Field(
        default='tools.ozone.moderation.defs#modEventView', alias='$type', frozen=True
    )


class ModEventViewDetail(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`."""

    created_at: string_formats.DateTime  #: Created at.
    created_by: string_formats.Did  #: Created by.
    event: te.Annotated[
        t.Union[
            'models.ToolsOzoneModerationDefs.ModEventTakedown',
            'models.ToolsOzoneModerationDefs.ModEventReverseTakedown',
            'models.ToolsOzoneModerationDefs.ModEventComment',
            'models.ToolsOzoneModerationDefs.ModEventReport',
            'models.ToolsOzoneModerationDefs.ModEventLabel',
            'models.ToolsOzoneModerationDefs.ModEventAcknowledge',
            'models.ToolsOzoneModerationDefs.ModEventEscalate',
            'models.ToolsOzoneModerationDefs.ModEventMute',
            'models.ToolsOzoneModerationDefs.ModEventUnmute',
            'models.ToolsOzoneModerationDefs.ModEventMuteReporter',
            'models.ToolsOzoneModerationDefs.ModEventUnmuteReporter',
            'models.ToolsOzoneModerationDefs.ModEventEmail',
            'models.ToolsOzoneModerationDefs.ModEventResolveAppeal',
            'models.ToolsOzoneModerationDefs.ModEventDivert',
            'models.ToolsOzoneModerationDefs.ModEventTag',
            'models.ToolsOzoneModerationDefs.AccountEvent',
            'models.ToolsOzoneModerationDefs.IdentityEvent',
            'models.ToolsOzoneModerationDefs.RecordEvent',
            'models.ToolsOzoneModerationDefs.ModEventPriorityScore',
        ],
        Field(discriminator='py_type'),
    ]  #: Event.
    id: int  #: Id.
    subject: te.Annotated[
        t.Union[
            'models.ToolsOzoneModerationDefs.RepoView',
            'models.ToolsOzoneModerationDefs.RepoViewNotFound',
            'models.ToolsOzoneModerationDefs.RecordView',
            'models.ToolsOzoneModerationDefs.RecordViewNotFound',
        ],
        Field(discriminator='py_type'),
    ]  #: Subject.
    subject_blobs: t.List['models.ToolsOzoneModerationDefs.BlobView']  #: Subject blobs.

    py_type: t.Literal['tools.ozone.moderation.defs#modEventViewDetail'] = Field(
        default='tools.ozone.moderation.defs#modEventViewDetail', alias='$type', frozen=True
    )


class SubjectStatusView(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`."""

    created_at: (
        string_formats.DateTime
    )  #: Timestamp referencing the first moderation status impacting event was emitted on the subject.
    id: int  #: Id.
    review_state: 'models.ToolsOzoneModerationDefs.SubjectReviewState'  #: Review state.
    subject: te.Annotated[
        t.Union['models.ComAtprotoAdminDefs.RepoRef', 'models.ComAtprotoRepoStrongRef.Main'],
        Field(discriminator='py_type'),
    ]  #: Subject.
    updated_at: (
        string_formats.DateTime
    )  #: Timestamp referencing when the last update was made to the moderation status of the subject.
    account_stats: t.Optional['models.ToolsOzoneModerationDefs.AccountStats'] = (
        None  #: Statistics related to the account subject.
    )
    appealed: t.Optional[bool] = (
        None  #: True indicates that the a previously taken moderator action was appealed against, by the author of the content. False indicates last appeal was resolved by moderators.
    )
    comment: t.Optional[str] = None  #: Sticky comment on the subject.
    hosting: t.Optional[
        te.Annotated[
            t.Union['models.ToolsOzoneModerationDefs.AccountHosting', 'models.ToolsOzoneModerationDefs.RecordHosting'],
            Field(default=None, discriminator='py_type'),
        ]
    ] = None  #: Hosting.
    last_appealed_at: t.Optional[string_formats.DateTime] = (
        None  #: Timestamp referencing when the author of the subject appealed a moderation action.
    )
    last_reported_at: t.Optional[string_formats.DateTime] = None  #: Last reported at.
    last_reviewed_at: t.Optional[string_formats.DateTime] = None  #: Last reviewed at.
    last_reviewed_by: t.Optional[string_formats.Did] = None  #: Last reviewed by.
    mute_reporting_until: t.Optional[string_formats.DateTime] = None  #: Mute reporting until.
    mute_until: t.Optional[string_formats.DateTime] = None  #: Mute until.
    priority_score: t.Optional[int] = Field(
        default=None, ge=0, le=100
    )  #: Numeric value representing the level of priority. Higher score means higher priority.
    records_stats: t.Optional['models.ToolsOzoneModerationDefs.RecordsStats'] = (
        None  #: Statistics related to the record subjects authored by the subject's account.
    )
    subject_blob_cids: t.Optional[t.List[string_formats.Cid]] = None  #: Subject blob cids.
    subject_repo_handle: t.Optional[str] = None  #: Subject repo handle.
    suspend_until: t.Optional[string_formats.DateTime] = None  #: Suspend until.
    tags: t.Optional[t.List[str]] = None  #: Tags.
    takendown: t.Optional[bool] = None  #: Takendown.

    py_type: t.Literal['tools.ozone.moderation.defs#subjectStatusView'] = Field(
        default='tools.ozone.moderation.defs#subjectStatusView', alias='$type', frozen=True
    )


class SubjectView(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`. Detailed view of a subject. For record subjects, the author's repo and profile will be returned."""

    subject: str  #: Subject.
    type: 'models.ComAtprotoModerationDefs.SubjectType'  #: Type.
    profile: t.Optional[te.Annotated[t.Union['base.UnknownUnionModel'], Field(default=None)]] = None  #: Profile.
    record: t.Optional['models.ToolsOzoneModerationDefs.RecordViewDetail'] = None  #: Record.
    repo: t.Optional['models.ToolsOzoneModerationDefs.RepoViewDetail'] = None  #: Repo.
    status: t.Optional['models.ToolsOzoneModerationDefs.SubjectStatusView'] = None  #: Status.

    py_type: t.Literal['tools.ozone.moderation.defs#subjectView'] = Field(
        default='tools.ozone.moderation.defs#subjectView', alias='$type', frozen=True
    )


class AccountStats(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`. Statistics about a particular account subject."""

    appeal_count: t.Optional[int] = None  #: Total number of appeals against a moderation action on the account.
    escalate_count: t.Optional[int] = None  #: Number of times the account was escalated.
    report_count: t.Optional[int] = None  #: Total number of reports on the account.
    suspend_count: t.Optional[int] = None  #: Number of times the account was suspended.
    takedown_count: t.Optional[int] = None  #: Number of times the account was taken down.

    py_type: t.Literal['tools.ozone.moderation.defs#accountStats'] = Field(
        default='tools.ozone.moderation.defs#accountStats', alias='$type', frozen=True
    )


class RecordsStats(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`. Statistics about a set of record subject items."""

    appealed_count: t.Optional[int] = None  #: Number of items that were appealed at least once.
    escalated_count: t.Optional[int] = None  #: Number of items that were escalated at least once.
    pending_count: t.Optional[int] = None  #: Number of item currently in "reviewOpen" or "reviewEscalated" state.
    processed_count: t.Optional[int] = None  #: Number of item currently in "reviewNone" or "reviewClosed" state.
    reported_count: t.Optional[int] = None  #: Number of items that were reported at least once.
    subject_count: t.Optional[int] = None  #: Total number of item in the set.
    takendown_count: t.Optional[int] = None  #: Number of item currently taken down.
    total_reports: t.Optional[int] = None  #: Cumulative sum of the number of reports on the items in the set.

    py_type: t.Literal['tools.ozone.moderation.defs#recordsStats'] = Field(
        default='tools.ozone.moderation.defs#recordsStats', alias='$type', frozen=True
    )


SubjectReviewState = t.Union[
    'models.ToolsOzoneModerationDefs.ReviewOpen',
    'models.ToolsOzoneModerationDefs.ReviewEscalated',
    'models.ToolsOzoneModerationDefs.ReviewClosed',
    'models.ToolsOzoneModerationDefs.ReviewNone',
    str,
]  #: Subject review state

ReviewOpen = t.Literal[
    'tools.ozone.moderation.defs#reviewOpen'
]  #: Moderator review status of a subject: Open. Indicates that the subject needs to be reviewed by a moderator

ReviewEscalated = t.Literal[
    'tools.ozone.moderation.defs#reviewEscalated'
]  #: Moderator review status of a subject: Escalated. Indicates that the subject was escalated for review by a moderator

ReviewClosed = t.Literal[
    'tools.ozone.moderation.defs#reviewClosed'
]  #: Moderator review status of a subject: Closed. Indicates that the subject was already reviewed and resolved by a moderator

ReviewNone = t.Literal[
    'tools.ozone.moderation.defs#reviewNone'
]  #: Moderator review status of a subject: Unnecessary. Indicates that the subject does not need a review at the moment but there is probably some moderation related metadata available for it


class ModEventTakedown(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`. Take down a subject permanently or temporarily."""

    acknowledge_account_subjects: t.Optional[bool] = (
        None  #: If true, all other reports on content authored by this account will be resolved (acknowledged).
    )
    comment: t.Optional[str] = None  #: Comment.
    duration_in_hours: t.Optional[int] = (
        None  #: Indicates how long the takedown should be in effect before automatically expiring.
    )
    policies: t.Optional[t.List[str]] = Field(
        default=None, max_length=5
    )  #: Names/Keywords of the policies that drove the decision.

    py_type: t.Literal['tools.ozone.moderation.defs#modEventTakedown'] = Field(
        default='tools.ozone.moderation.defs#modEventTakedown', alias='$type', frozen=True
    )


class ModEventReverseTakedown(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`. Revert take down action on a subject."""

    comment: t.Optional[str] = None  #: Describe reasoning behind the reversal.

    py_type: t.Literal['tools.ozone.moderation.defs#modEventReverseTakedown'] = Field(
        default='tools.ozone.moderation.defs#modEventReverseTakedown', alias='$type', frozen=True
    )


class ModEventResolveAppeal(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`. Resolve appeal on a subject."""

    comment: t.Optional[str] = None  #: Describe resolution.

    py_type: t.Literal['tools.ozone.moderation.defs#modEventResolveAppeal'] = Field(
        default='tools.ozone.moderation.defs#modEventResolveAppeal', alias='$type', frozen=True
    )


class ModEventComment(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`. Add a comment to a subject. An empty comment will clear any previously set sticky comment."""

    comment: t.Optional[str] = None  #: Comment.
    sticky: t.Optional[bool] = None  #: Make the comment persistent on the subject.

    py_type: t.Literal['tools.ozone.moderation.defs#modEventComment'] = Field(
        default='tools.ozone.moderation.defs#modEventComment', alias='$type', frozen=True
    )


class ModEventReport(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`. Report a subject."""

    report_type: 'models.ComAtprotoModerationDefs.ReasonType'  #: Report type.
    comment: t.Optional[str] = None  #: Comment.
    is_reporter_muted: t.Optional[bool] = (
        None  #: Set to true if the reporter was muted from reporting at the time of the event. These reports won't impact the reviewState of the subject.
    )

    py_type: t.Literal['tools.ozone.moderation.defs#modEventReport'] = Field(
        default='tools.ozone.moderation.defs#modEventReport', alias='$type', frozen=True
    )


class ModEventLabel(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`. Apply/Negate labels on a subject."""

    create_label_vals: t.List[str]  #: Create label vals.
    negate_label_vals: t.List[str]  #: Negate label vals.
    comment: t.Optional[str] = None  #: Comment.
    duration_in_hours: t.Optional[int] = (
        None  #: Indicates how long the label will remain on the subject. Only applies on labels that are being added.
    )

    py_type: t.Literal['tools.ozone.moderation.defs#modEventLabel'] = Field(
        default='tools.ozone.moderation.defs#modEventLabel', alias='$type', frozen=True
    )


class ModEventPriorityScore(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`. Set priority score of the subject. Higher score means higher priority."""

    score: int = Field(ge=0, le=100)  #: Score.
    comment: t.Optional[str] = None  #: Comment.

    py_type: t.Literal['tools.ozone.moderation.defs#modEventPriorityScore'] = Field(
        default='tools.ozone.moderation.defs#modEventPriorityScore', alias='$type', frozen=True
    )


class ModEventAcknowledge(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`."""

    acknowledge_account_subjects: t.Optional[bool] = (
        None  #: If true, all other reports on content authored by this account will be resolved (acknowledged).
    )
    comment: t.Optional[str] = None  #: Comment.

    py_type: t.Literal['tools.ozone.moderation.defs#modEventAcknowledge'] = Field(
        default='tools.ozone.moderation.defs#modEventAcknowledge', alias='$type', frozen=True
    )


class ModEventEscalate(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`."""

    comment: t.Optional[str] = None  #: Comment.

    py_type: t.Literal['tools.ozone.moderation.defs#modEventEscalate'] = Field(
        default='tools.ozone.moderation.defs#modEventEscalate', alias='$type', frozen=True
    )


class ModEventMute(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`. Mute incoming reports on a subject."""

    duration_in_hours: int  #: Indicates how long the subject should remain muted.
    comment: t.Optional[str] = None  #: Comment.

    py_type: t.Literal['tools.ozone.moderation.defs#modEventMute'] = Field(
        default='tools.ozone.moderation.defs#modEventMute', alias='$type', frozen=True
    )


class ModEventUnmute(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`. Unmute action on a subject."""

    comment: t.Optional[str] = None  #: Describe reasoning behind the reversal.

    py_type: t.Literal['tools.ozone.moderation.defs#modEventUnmute'] = Field(
        default='tools.ozone.moderation.defs#modEventUnmute', alias='$type', frozen=True
    )


class ModEventMuteReporter(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`. Mute incoming reports from an account."""

    comment: t.Optional[str] = None  #: Comment.
    duration_in_hours: t.Optional[int] = (
        None  #: Indicates how long the account should remain muted. Falsy value here means a permanent mute.
    )

    py_type: t.Literal['tools.ozone.moderation.defs#modEventMuteReporter'] = Field(
        default='tools.ozone.moderation.defs#modEventMuteReporter', alias='$type', frozen=True
    )


class ModEventUnmuteReporter(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`. Unmute incoming reports from an account."""

    comment: t.Optional[str] = None  #: Describe reasoning behind the reversal.

    py_type: t.Literal['tools.ozone.moderation.defs#modEventUnmuteReporter'] = Field(
        default='tools.ozone.moderation.defs#modEventUnmuteReporter', alias='$type', frozen=True
    )


class ModEventEmail(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`. Keep a log of outgoing email to a user."""

    subject_line: str  #: The subject line of the email sent to the user.
    comment: t.Optional[str] = None  #: Additional comment about the outgoing comm.
    content: t.Optional[str] = None  #: The content of the email sent to the user.

    py_type: t.Literal['tools.ozone.moderation.defs#modEventEmail'] = Field(
        default='tools.ozone.moderation.defs#modEventEmail', alias='$type', frozen=True
    )


class ModEventDivert(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`. Divert a record's blobs to a 3rd party service for further scanning/tagging."""

    comment: t.Optional[str] = None  #: Comment.

    py_type: t.Literal['tools.ozone.moderation.defs#modEventDivert'] = Field(
        default='tools.ozone.moderation.defs#modEventDivert', alias='$type', frozen=True
    )


class ModEventTag(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`. Add/Remove a tag on a subject."""

    add: t.List[str]  #: Tags to be added to the subject. If already exists, won't be duplicated.
    remove: t.List[str]  #: Tags to be removed to the subject. Ignores a tag If it doesn't exist, won't be duplicated.
    comment: t.Optional[str] = None  #: Additional comment about added/removed tags.

    py_type: t.Literal['tools.ozone.moderation.defs#modEventTag'] = Field(
        default='tools.ozone.moderation.defs#modEventTag', alias='$type', frozen=True
    )


class AccountEvent(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`. Logs account status related events on a repo subject. Normally captured by automod from the firehose and emitted to ozone for historical tracking."""

    active: (
        bool  #: Indicates that the account has a repository which can be fetched from the host that emitted this event.
    )
    timestamp: string_formats.DateTime  #: Timestamp.
    comment: t.Optional[str] = None  #: Comment.
    status: t.Optional[
        t.Union[
            t.Literal['unknown'],
            t.Literal['deactivated'],
            t.Literal['deleted'],
            t.Literal['takendown'],
            t.Literal['suspended'],
            t.Literal['tombstoned'],
            str,
        ]
    ] = None  #: Status.

    py_type: t.Literal['tools.ozone.moderation.defs#accountEvent'] = Field(
        default='tools.ozone.moderation.defs#accountEvent', alias='$type', frozen=True
    )


class IdentityEvent(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`. Logs identity related events on a repo subject. Normally captured by automod from the firehose and emitted to ozone for historical tracking."""

    timestamp: string_formats.DateTime  #: Timestamp.
    comment: t.Optional[str] = None  #: Comment.
    handle: t.Optional[string_formats.Handle] = None  #: Handle.
    pds_host: t.Optional[string_formats.Uri] = None  #: Pds host.
    tombstone: t.Optional[bool] = None  #: Tombstone.

    py_type: t.Literal['tools.ozone.moderation.defs#identityEvent'] = Field(
        default='tools.ozone.moderation.defs#identityEvent', alias='$type', frozen=True
    )


class RecordEvent(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`. Logs lifecycle event on a record subject. Normally captured by automod from the firehose and emitted to ozone for historical tracking."""

    op: t.Union[t.Literal['create'], t.Literal['update'], t.Literal['delete'], str]  #: Op.
    timestamp: string_formats.DateTime  #: Timestamp.
    cid: t.Optional[string_formats.Cid] = None  #: Cid.
    comment: t.Optional[str] = None  #: Comment.

    py_type: t.Literal['tools.ozone.moderation.defs#recordEvent'] = Field(
        default='tools.ozone.moderation.defs#recordEvent', alias='$type', frozen=True
    )


class RepoView(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`."""

    did: string_formats.Did  #: Did.
    handle: string_formats.Handle  #: Handle.
    indexed_at: string_formats.DateTime  #: Indexed at.
    moderation: 'models.ToolsOzoneModerationDefs.Moderation'  #: Moderation.
    related_records: t.List['UnknownType']  #: Related records.
    deactivated_at: t.Optional[string_formats.DateTime] = None  #: Deactivated at.
    email: t.Optional[str] = None  #: Email.
    invite_note: t.Optional[str] = None  #: Invite note.
    invited_by: t.Optional['models.ComAtprotoServerDefs.InviteCode'] = None  #: Invited by.
    invites_disabled: t.Optional[bool] = None  #: Invites disabled.
    threat_signatures: t.Optional[t.List['models.ComAtprotoAdminDefs.ThreatSignature']] = None  #: Threat signatures.

    py_type: t.Literal['tools.ozone.moderation.defs#repoView'] = Field(
        default='tools.ozone.moderation.defs#repoView', alias='$type', frozen=True
    )


class RepoViewDetail(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`."""

    did: string_formats.Did  #: Did.
    handle: string_formats.Handle  #: Handle.
    indexed_at: string_formats.DateTime  #: Indexed at.
    moderation: 'models.ToolsOzoneModerationDefs.ModerationDetail'  #: Moderation.
    related_records: t.List['UnknownType']  #: Related records.
    deactivated_at: t.Optional[string_formats.DateTime] = None  #: Deactivated at.
    email: t.Optional[str] = None  #: Email.
    email_confirmed_at: t.Optional[string_formats.DateTime] = None  #: Email confirmed at.
    invite_note: t.Optional[str] = None  #: Invite note.
    invited_by: t.Optional['models.ComAtprotoServerDefs.InviteCode'] = None  #: Invited by.
    invites: t.Optional[t.List['models.ComAtprotoServerDefs.InviteCode']] = None  #: Invites.
    invites_disabled: t.Optional[bool] = None  #: Invites disabled.
    labels: t.Optional[t.List['models.ComAtprotoLabelDefs.Label']] = None  #: Labels.
    threat_signatures: t.Optional[t.List['models.ComAtprotoAdminDefs.ThreatSignature']] = None  #: Threat signatures.

    py_type: t.Literal['tools.ozone.moderation.defs#repoViewDetail'] = Field(
        default='tools.ozone.moderation.defs#repoViewDetail', alias='$type', frozen=True
    )


class RepoViewNotFound(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`."""

    did: string_formats.Did  #: Did.

    py_type: t.Literal['tools.ozone.moderation.defs#repoViewNotFound'] = Field(
        default='tools.ozone.moderation.defs#repoViewNotFound', alias='$type', frozen=True
    )


class RecordView(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`."""

    blob_cids: t.List[string_formats.Cid]  #: Blob cids.
    cid: string_formats.Cid  #: Cid.
    indexed_at: string_formats.DateTime  #: Indexed at.
    moderation: 'models.ToolsOzoneModerationDefs.Moderation'  #: Moderation.
    repo: 'models.ToolsOzoneModerationDefs.RepoView'  #: Repo.
    uri: string_formats.AtUri  #: Uri.
    value: 'UnknownType'  #: Value.

    py_type: t.Literal['tools.ozone.moderation.defs#recordView'] = Field(
        default='tools.ozone.moderation.defs#recordView', alias='$type', frozen=True
    )


class RecordViewDetail(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`."""

    blobs: t.List['models.ToolsOzoneModerationDefs.BlobView']  #: Blobs.
    cid: string_formats.Cid  #: Cid.
    indexed_at: string_formats.DateTime  #: Indexed at.
    moderation: 'models.ToolsOzoneModerationDefs.ModerationDetail'  #: Moderation.
    repo: 'models.ToolsOzoneModerationDefs.RepoView'  #: Repo.
    uri: string_formats.AtUri  #: Uri.
    value: 'UnknownType'  #: Value.
    labels: t.Optional[t.List['models.ComAtprotoLabelDefs.Label']] = None  #: Labels.

    py_type: t.Literal['tools.ozone.moderation.defs#recordViewDetail'] = Field(
        default='tools.ozone.moderation.defs#recordViewDetail', alias='$type', frozen=True
    )


class RecordViewNotFound(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`."""

    uri: string_formats.AtUri  #: Uri.

    py_type: t.Literal['tools.ozone.moderation.defs#recordViewNotFound'] = Field(
        default='tools.ozone.moderation.defs#recordViewNotFound', alias='$type', frozen=True
    )


class Moderation(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`."""

    subject_status: t.Optional['models.ToolsOzoneModerationDefs.SubjectStatusView'] = None  #: Subject status.

    py_type: t.Literal['tools.ozone.moderation.defs#moderation'] = Field(
        default='tools.ozone.moderation.defs#moderation', alias='$type', frozen=True
    )


class ModerationDetail(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`."""

    subject_status: t.Optional['models.ToolsOzoneModerationDefs.SubjectStatusView'] = None  #: Subject status.

    py_type: t.Literal['tools.ozone.moderation.defs#moderationDetail'] = Field(
        default='tools.ozone.moderation.defs#moderationDetail', alias='$type', frozen=True
    )


class BlobView(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`."""

    cid: string_formats.Cid  #: Cid.
    created_at: string_formats.DateTime  #: Created at.
    mime_type: str  #: Mime type.
    size: int  #: Size.
    details: t.Optional[
        te.Annotated[
            t.Union['models.ToolsOzoneModerationDefs.ImageDetails', 'models.ToolsOzoneModerationDefs.VideoDetails'],
            Field(default=None, discriminator='py_type'),
        ]
    ] = None  #: Details.
    moderation: t.Optional['models.ToolsOzoneModerationDefs.Moderation'] = None  #: Moderation.

    py_type: t.Literal['tools.ozone.moderation.defs#blobView'] = Field(
        default='tools.ozone.moderation.defs#blobView', alias='$type', frozen=True
    )


class ImageDetails(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`."""

    height: int  #: Height.
    width: int  #: Width.

    py_type: t.Literal['tools.ozone.moderation.defs#imageDetails'] = Field(
        default='tools.ozone.moderation.defs#imageDetails', alias='$type', frozen=True
    )


class VideoDetails(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`."""

    height: int  #: Height.
    length: int  #: Length.
    width: int  #: Width.

    py_type: t.Literal['tools.ozone.moderation.defs#videoDetails'] = Field(
        default='tools.ozone.moderation.defs#videoDetails', alias='$type', frozen=True
    )


class AccountHosting(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`."""

    status: t.Union[
        t.Literal['takendown'],
        t.Literal['suspended'],
        t.Literal['deleted'],
        t.Literal['deactivated'],
        t.Literal['unknown'],
        str,
    ]  #: Status.
    created_at: t.Optional[string_formats.DateTime] = None  #: Created at.
    deactivated_at: t.Optional[string_formats.DateTime] = None  #: Deactivated at.
    deleted_at: t.Optional[string_formats.DateTime] = None  #: Deleted at.
    reactivated_at: t.Optional[string_formats.DateTime] = None  #: Reactivated at.
    updated_at: t.Optional[string_formats.DateTime] = None  #: Updated at.

    py_type: t.Literal['tools.ozone.moderation.defs#accountHosting'] = Field(
        default='tools.ozone.moderation.defs#accountHosting', alias='$type', frozen=True
    )


class RecordHosting(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`."""

    status: t.Union[t.Literal['deleted'], t.Literal['unknown'], str]  #: Status.
    created_at: t.Optional[string_formats.DateTime] = None  #: Created at.
    deleted_at: t.Optional[string_formats.DateTime] = None  #: Deleted at.
    updated_at: t.Optional[string_formats.DateTime] = None  #: Updated at.

    py_type: t.Literal['tools.ozone.moderation.defs#recordHosting'] = Field(
        default='tools.ozone.moderation.defs#recordHosting', alias='$type', frozen=True
    )


class ReporterStats(base.ModelBase):
    """Definition model for :obj:`tools.ozone.moderation.defs`."""

    account_report_count: int  #: The total number of reports made by the user on accounts.
    did: string_formats.Did  #: Did.
    labeled_account_count: int  #: The total number of accounts labeled as a result of the user's reports.
    labeled_record_count: int  #: The total number of records labeled as a result of the user's reports.
    record_report_count: int  #: The total number of reports made by the user on records.
    reported_account_count: int  #: The total number of accounts reported by the user.
    reported_record_count: int  #: The total number of records reported by the user.
    takendown_account_count: int  #: The total number of accounts taken down as a result of the user's reports.
    takendown_record_count: int  #: The total number of records taken down as a result of the user's reports.

    py_type: t.Literal['tools.ozone.moderation.defs#reporterStats'] = Field(
        default='tools.ozone.moderation.defs#reporterStats', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`app.bsky.feed.getPostThread`."""

    uri: string_formats.AtUri  #: Reference (AT-URI) to post record.
    depth: t.Optional[int] = Field(
        default=6, ge=0, le=1000
    )  #: How many levels of reply depth should be included in response.
    parent_height: t.Optional[int] = Field(
        default=80, ge=0, le=1000
    )  #: How many levels of parent (and grandparent, etc) post to include.


class ParamsDict(t.TypedDict):
    uri: string_formats.AtUri  #: Reference (AT-URI) to post record.
    depth: te.NotRequired[t.Optional[int]]  #: How many levels of reply depth should be included in response.
    parent_height: te.NotRequired[t.Optional[int]]  #: How many levels of parent (and grandparent, etc) post to include.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`app.bsky.feed.getPostThread`."""

    thread: te.Annotated[
        t.Union[
            'models.AppBskyFeedDefs.ThreadViewPost',
            'models.AppBskyFeedDefs.NotFoundPost',
            'models.AppBskyFeedDefs.BlockedPost',
        ],
        Field(discriminator='py_type'),
    ]  #: Thread.
    threadgate: t.Optional['models.AppBskyFeedDefs.ThreadgateView'] = None  #: Threadgate.



from atproto_client import AsyncClient, Client, Session, SessionEvent, models
from atproto_client import utils as client_utils
from atproto_client.request import AsyncRequest, Request
from atproto_core.car import CAR
from atproto_core.cid import CID, CIDType
from atproto_core.did_doc import DidDocument
from atproto_core.nsid import NSID
from atproto_core.uri import AtUri
from atproto_crypto.did import Multikey, get_did_key
from atproto_crypto.multibase import bytes_to_multibase, multibase_to_bytes
from atproto_crypto.verify import verify_signature
from atproto_firehose import (
    AsyncFirehoseSubscribeLabelsClient,
    AsyncFirehoseSubscribeReposClient,
    FirehoseSubscribeLabelsClient,
    FirehoseSubscribeReposClient,
    parse_subscribe_labels_message,
    parse_subscribe_repos_message,
)
from atproto_firehose import models as firehose_models
from atproto_identity.cache.in_memory_cache import AsyncDidInMemoryCache, DidInMemoryCache
from atproto_identity.did.atproto_data import AtprotoData
from atproto_identity.resolver import AsyncIdResolver, IdResolver
from atproto_server.auth.jwt import (
    JwtPayload,
    decode_jwt_payload,
    get_jwt_payload,
    parse_jwt,
    validate_jwt_payload,
    verify_jwt,
    verify_jwt_async,
)

__all__ = [
    # client
    'AsyncClient',
    'Client',
    'SessionEvent',
    'Session',
    'client_utils',
    'models',
    'Request',
    'AsyncRequest',
    # core
    'CAR',
    'CID',
    'CIDType',
    'DidDocument',
    'NSID',
    'AtUri',
    # crypto
    'bytes_to_multibase',
    'get_did_key',
    'multibase_to_bytes',
    'Multikey',
    'verify_signature',
    # firehose
    'AsyncFirehoseSubscribeLabelsClient',
    'AsyncFirehoseSubscribeReposClient',
    'FirehoseSubscribeLabelsClient',
    'FirehoseSubscribeReposClient',
    'parse_subscribe_labels_message',
    'parse_subscribe_repos_message',
    'firehose_models',
    # identity
    'AtprotoData',
    'AsyncDidInMemoryCache',
    'DidInMemoryCache',
    'AsyncIdResolver',
    'IdResolver',
    # server jwt
    'JwtPayload',
    'decode_jwt_payload',
    'get_jwt_payload',
    'parse_jwt',
    'validate_jwt_payload',
    'verify_jwt',
    'verify_jwt_async',
]



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Response(base.ResponseModelBase):
    """Output data model for :obj:`tools.ozone.server.getConfig`."""

    appview: t.Optional['models.ToolsOzoneServerGetConfig.ServiceConfig'] = None  #: Appview.
    blob_divert: t.Optional['models.ToolsOzoneServerGetConfig.ServiceConfig'] = None  #: Blob divert.
    chat: t.Optional['models.ToolsOzoneServerGetConfig.ServiceConfig'] = None  #: Chat.
    pds: t.Optional['models.ToolsOzoneServerGetConfig.ServiceConfig'] = None  #: Pds.
    viewer: t.Optional['models.ToolsOzoneServerGetConfig.ViewerConfig'] = None  #: Viewer.


class ServiceConfig(base.ModelBase):
    """Definition model for :obj:`tools.ozone.server.getConfig`."""

    url: t.Optional[string_formats.Uri] = None  #: Url.

    py_type: t.Literal['tools.ozone.server.getConfig#serviceConfig'] = Field(
        default='tools.ozone.server.getConfig#serviceConfig', alias='$type', frozen=True
    )


class ViewerConfig(base.ModelBase):
    """Definition model for :obj:`tools.ozone.server.getConfig`."""

    role: t.Optional[
        t.Union[
            'models.ToolsOzoneTeamDefs.RoleAdmin',
            'models.ToolsOzoneTeamDefs.RoleModerator',
            'models.ToolsOzoneTeamDefs.RoleTriage',
            str,
        ]
    ] = None  #: Role.

    py_type: t.Literal['tools.ozone.server.getConfig#viewerConfig'] = Field(
        default='tools.ozone.server.getConfig#viewerConfig', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client.models.unknown_type import UnknownType
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`com.atproto.identity.resolveDid`."""

    did: string_formats.Did  #: DID to resolve.


class ParamsDict(t.TypedDict):
    did: string_formats.Did  #: DID to resolve.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.identity.resolveDid`."""

    did_doc: 'UnknownType'  #: The complete DID document for the identity.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Record(base.RecordModelBase):
    """Record model for :obj:`com.atproto.lexicon.schema`."""

    lexicon: int  #: Indicates the 'version' of the Lexicon language. Must be '1' for the current atproto/Lexicon schema system.

    py_type: t.Literal['com.atproto.lexicon.schema'] = Field(
        default='com.atproto.lexicon.schema', alias='$type', frozen=True
    )


class GetRecordResponse(base.SugarResponseModelBase):
    """Get record response for :obj:`models.ComAtprotoLexiconSchema.Record`."""

    uri: str  #: The URI of the record.
    value: 'models.ComAtprotoLexiconSchema.Record'  #: The record.
    cid: t.Optional[str] = None  #: The CID of the record.


class ListRecordsResponse(base.SugarResponseModelBase):
    """List records response for :obj:`models.ComAtprotoLexiconSchema.Record`."""

    records: t.Dict[str, 'models.ComAtprotoLexiconSchema.Record']  #: Map of URIs to records.
    cursor: t.Optional[str] = None  #: Next page cursor.


class CreateRecordResponse(base.SugarResponseModelBase):
    """Create record response for :obj:`models.ComAtprotoLexiconSchema.Record`."""

    uri: str  #: The URI of the record.
    cid: str  #: The CID of the record.



import typing as t

import libipld

from atproto_core.cid import CID
from atproto_core.exceptions import InvalidCARFile

Blocks = t.Dict[CID, dict]


class CAR:
    """CAR file."""

    def __init__(self, root: CID, blocks: Blocks) -> None:
        self._root = root
        self._blocks = blocks

    @property
    def root(self) -> CID:
        """Get root."""
        return self._root

    @property
    def blocks(self) -> Blocks:
        """Get blocks."""
        return self._blocks

    @classmethod
    def from_bytes(cls, data: bytes) -> 'CAR':
        """Decode CAR file.

        Note:
            You could pass as `data` response of `client.com.atproto.sync.get_repo`, for example.
            And other responses of methods in the `sync` namespace.

        Example:
            >>> from atproto import CAR, Client
            >>> client = Client()
            >>> client.login('my-handle', 'my-password')
            >>> repo = client.com.atproto.sync.get_repo({'did': client.me.did})
            >>> car_file = CAR.from_bytes(repo)
            >>> print(car_file.root)
            >>> print(car_file.blocks)

        Args:
            data: Content of the CAR file.

        Returns:
            :obj:`atproto.CAR`: Parsed CAR file.
        """
        header, blocks = libipld.decode_car(data)

        roots = header.get('roots')
        if isinstance(roots, list) and len(roots):
            # The first element of the CAR roots metadata array must be the CID of the most relevant Commit object.
            # For a generic export, this is the current (most recent) commit.
            # Additional CIDs may also be present in the roots array, with (for now) undefined meaning or order.
            root: CID = CID.decode(roots[0])
        else:
            raise InvalidCARFile('Invalid CAR file. Expected at least one root.')

        blocks = {CID.decode(cid): block for cid, block in blocks.items()}
        return cls(root=root, blocks=blocks)



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Record(base.RecordModelBase):
    """Record model for :obj:`app.bsky.graph.verification`."""

    created_at: string_formats.DateTime  #: Date of when the verification was created.
    display_name: str  #: Display name of the subject the verification applies to at the moment of verifying, which might not be the same at the time of viewing. The verification is only valid if the current displayName matches the one at the time of verifying.
    handle: string_formats.Handle  #: Handle of the subject the verification applies to at the moment of verifying, which might not be the same at the time of viewing. The verification is only valid if the current handle matches the one at the time of verifying.
    subject: string_formats.Did  #: DID of the subject the verification applies to.

    py_type: t.Literal['app.bsky.graph.verification'] = Field(
        default='app.bsky.graph.verification', alias='$type', frozen=True
    )


class GetRecordResponse(base.SugarResponseModelBase):
    """Get record response for :obj:`models.AppBskyGraphVerification.Record`."""

    uri: str  #: The URI of the record.
    value: 'models.AppBskyGraphVerification.Record'  #: The record.
    cid: t.Optional[str] = None  #: The CID of the record.


class ListRecordsResponse(base.SugarResponseModelBase):
    """List records response for :obj:`models.AppBskyGraphVerification.Record`."""

    records: t.Dict[str, 'models.AppBskyGraphVerification.Record']  #: Map of URIs to records.
    cursor: t.Optional[str] = None  #: Next page cursor.


class CreateRecordResponse(base.SugarResponseModelBase):
    """Create record response for :obj:`models.AppBskyGraphVerification.Record`."""

    uri: str  #: The URI of the record.
    cid: str  #: The CID of the record.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`app.bsky.graph.getRelationships`."""

    actor: string_formats.AtIdentifier  #: Primary account requesting relationships for.
    others: t.Optional[t.List[string_formats.AtIdentifier]] = Field(
        default=None, max_length=30
    )  #: List of 'other' accounts to be related back to the primary.


class ParamsDict(t.TypedDict):
    actor: string_formats.AtIdentifier  #: Primary account requesting relationships for.
    others: te.NotRequired[
        t.Optional[t.List[string_formats.AtIdentifier]]
    ]  #: List of 'other' accounts to be related back to the primary.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`app.bsky.graph.getRelationships`."""

    relationships: t.List[
        te.Annotated[
            t.Union['models.AppBskyGraphDefs.Relationship', 'models.AppBskyGraphDefs.NotFoundActor'],
            Field(discriminator='py_type'),
        ]
    ]  #: Relationships.
    actor: t.Optional[string_formats.Did] = None  #: Actor.



import base64
import typing as t


def force_bytes(value: t.Union[bytes, str]) -> bytes:
    if isinstance(value, str):
        return value.encode('UTF-8')

    return value


def base64url_decode(input_data: t.Union[bytes, str]) -> bytes:
    input_bytes = force_bytes(input_data)

    rem = len(input_bytes) % 4

    if rem > 0:
        input_bytes += b'=' * (4 - rem)

    return base64.urlsafe_b64decode(input_bytes)


def base64url_encode(input_data: bytes) -> bytes:
    return base64.urlsafe_b64encode(input_data).replace(b'=', b'')



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Data(base.DataModelBase):
    """Input data model for :obj:`com.atproto.repo.deleteRecord`."""

    collection: string_formats.Nsid  #: The NSID of the record collection.
    repo: string_formats.AtIdentifier  #: The handle or DID of the repo (aka, current account).
    rkey: string_formats.RecordKey  #: The Record Key.
    swap_commit: t.Optional[string_formats.Cid] = None  #: Compare and swap with the previous commit by CID.
    swap_record: t.Optional[string_formats.Cid] = None  #: Compare and swap with the previous record by CID.


class DataDict(t.TypedDict):
    collection: string_formats.Nsid  #: The NSID of the record collection.
    repo: string_formats.AtIdentifier  #: The handle or DID of the repo (aka, current account).
    rkey: string_formats.RecordKey  #: The Record Key.
    swap_commit: te.NotRequired[t.Optional[string_formats.Cid]]  #: Compare and swap with the previous commit by CID.
    swap_record: te.NotRequired[t.Optional[string_formats.Cid]]  #: Compare and swap with the previous record by CID.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.repo.deleteRecord`."""

    commit: t.Optional['models.ComAtprotoRepoDefs.CommitMeta'] = None  #: Commit.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
    from atproto_client.models.blob_ref import BlobRef
from atproto_client.models import base


class Main(base.ModelBase):
    """Definition model for :obj:`app.bsky.embed.images`."""

    images: t.List['models.AppBskyEmbedImages.Image'] = Field(max_length=4)  #: Images.

    py_type: t.Literal['app.bsky.embed.images'] = Field(default='app.bsky.embed.images', alias='$type', frozen=True)


class Image(base.ModelBase):
    """Definition model for :obj:`app.bsky.embed.images`."""

    alt: str  #: Alt text description of the image, for accessibility.
    image: 'BlobRef'  #: Image.
    aspect_ratio: t.Optional['models.AppBskyEmbedDefs.AspectRatio'] = None  #: Aspect ratio.

    py_type: t.Literal['app.bsky.embed.images#image'] = Field(
        default='app.bsky.embed.images#image', alias='$type', frozen=True
    )


class View(base.ModelBase):
    """Definition model for :obj:`app.bsky.embed.images`."""

    images: t.List['models.AppBskyEmbedImages.ViewImage'] = Field(max_length=4)  #: Images.

    py_type: t.Literal['app.bsky.embed.images#view'] = Field(
        default='app.bsky.embed.images#view', alias='$type', frozen=True
    )


class ViewImage(base.ModelBase):
    """Definition model for :obj:`app.bsky.embed.images`."""

    alt: str  #: Alt text description of the image, for accessibility.
    fullsize: string_formats.Uri  #: Fully-qualified URL where a large version of the image can be fetched. May or may not be the exact original blob. For example, CDN location provided by the App View.
    thumb: string_formats.Uri  #: Fully-qualified URL where a thumbnail of the image can be fetched. For example, CDN location provided by the App View.
    aspect_ratio: t.Optional['models.AppBskyEmbedDefs.AspectRatio'] = None  #: Aspect ratio.

    py_type: t.Literal['app.bsky.embed.images#viewImage'] = Field(
        default='app.bsky.embed.images#viewImage', alias='$type', frozen=True
    )



import typing as t
from enum import Enum

import typing_extensions as te
from httpx import Headers

from atproto_client.models.utils import get_model_as_dict, get_model_as_json
from atproto_client.request import AsyncRequest, Request, Response

if t.TYPE_CHECKING:
    from atproto_client.models.base import DataModelBase, ParamsModelBase

# TODO(MarshalX): Generate async version automatically!


class InvokeType(Enum):
    QUERY = 'query'
    PROCEDURE = 'procedure'


_BASE_API_URL = 'https://bsky.social/xrpc'

_CONTENT_TYPE_JSON = 'application/json'
_DEFAULT_CONTENT_TYPE = _CONTENT_TYPE_JSON


def _handle_kwagrs(kwargs: dict) -> None:
    """Mutates input data."""
    headers = Headers(kwargs.get('headers'))  # case-insensitive dict

    content_type = headers.get('Content-Type', _DEFAULT_CONTENT_TYPE)
    # set content type from lexicon only if it's not set by user
    if 'Content-Type' not in headers and 'input_encoding' in kwargs:
        content_type = kwargs['input_encoding']
        headers['Content-Type'] = content_type

    if content_type == _CONTENT_TYPE_JSON and 'data' in kwargs and kwargs['data']:
        kwargs['data'] = get_model_as_json(kwargs['data'])

    if kwargs.get('params'):
        kwargs['params'] = get_model_as_dict(kwargs['params'])

    # set headers back
    kwargs['headers'] = dict(headers.items())

    # pop non-request kwargs
    kwargs.pop('input_encoding', None)
    kwargs.pop('output_encoding', None)


def _handle_base_url(base_url: t.Optional[str] = None) -> str:
    if base_url is None:
        return _BASE_API_URL

    if not base_url.endswith('/xrpc'):
        return f'{base_url.rstrip("/")}/xrpc'

    return base_url


class _ClientCommonMethodsMixin:
    def clone(self) -> te.Self:
        """Clone the client instance.

        Used to customize atproto proxy and set of labeler services.

        Returns:
            Cloned client instance.
        """
        return type(self)(base_url=self._base_url, request=self.request.clone())

    def update_base_url(self, base_url: t.Optional[str] = None) -> None:
        """Update XRPC base URL.

        Typically used for switching between PDSs.

        Args:
            base_url: New base URL.
                Defaults to bsky.social.
        """
        self._base_url = _handle_base_url(base_url)

    def _build_url(self, nsid: str) -> str:
        return f'{self._base_url}/{nsid}'


class ClientBase(_ClientCommonMethodsMixin):
    """Low-level methods are here."""

    def __init__(self, base_url: t.Optional[str] = None, request: t.Optional[Request] = None) -> None:
        if request is None:
            request = Request()

        self._request = request
        self._base_url = _handle_base_url(base_url)

    @property
    def request(self) -> Request:
        return self._request

    def invoke_query(
        self,
        nsid: str,
        params: t.Optional['ParamsModelBase'] = None,
        data: t.Optional[t.Union['DataModelBase', bytes]] = None,
        **kwargs: t.Any,
    ) -> Response:
        return self._invoke(InvokeType.QUERY, url=self._build_url(nsid), params=params, data=data, **kwargs)

    def invoke_procedure(
        self,
        nsid: str,
        params: t.Optional['ParamsModelBase'] = None,
        data: t.Optional[t.Union['DataModelBase', bytes]] = None,
        **kwargs: t.Any,
    ) -> Response:
        return self._invoke(InvokeType.PROCEDURE, url=self._build_url(nsid), params=params, data=data, **kwargs)

    def _invoke(self, invoke_type: InvokeType, **kwargs: t.Any) -> Response:
        _handle_kwagrs(kwargs)

        if invoke_type is InvokeType.QUERY:
            return self.request.get(**kwargs)
        return self.request.post(**kwargs)


class AsyncClientBase(_ClientCommonMethodsMixin):
    """Low-level methods are here."""

    def __init__(self, base_url: t.Optional[str] = None, request: t.Optional[AsyncRequest] = None) -> None:
        if request is None:
            request = AsyncRequest()

        self._request = request
        self._base_url = _handle_base_url(base_url)

    @property
    def request(self) -> AsyncRequest:
        return self._request

    async def invoke_query(
        self,
        nsid: str,
        params: t.Optional['ParamsModelBase'] = None,
        data: t.Optional[t.Union['DataModelBase', bytes]] = None,
        **kwargs: t.Any,
    ) -> Response:
        return await self._invoke(InvokeType.QUERY, url=self._build_url(nsid), params=params, data=data, **kwargs)

    async def invoke_procedure(
        self,
        nsid: str,
        params: t.Optional['ParamsModelBase'] = None,
        data: t.Optional[t.Union['DataModelBase', bytes]] = None,
        **kwargs: t.Any,
    ) -> Response:
        return await self._invoke(InvokeType.PROCEDURE, url=self._build_url(nsid), params=params, data=data, **kwargs)

    async def _invoke(self, invoke_type: InvokeType, **kwargs: t.Any) -> Response:
        _handle_kwagrs(kwargs)

        if invoke_type is InvokeType.QUERY:
            return await self.request.get(**kwargs)
        return await self.request.post(**kwargs)



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`chat.bsky.convo.getConvoAvailability`."""

    members: t.List[string_formats.Did] = Field(min_length=1, max_length=10)  #: Members.


class ParamsDict(t.TypedDict):
    members: t.List[string_formats.Did]  #: Members.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`chat.bsky.convo.getConvoAvailability`."""

    can_chat: bool  #: Can chat.
    convo: t.Optional['models.ChatBskyConvoDefs.ConvoView'] = None  #: Convo.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


from atproto_client.models import base, string_formats


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.server.checkAccountStatus`."""

    activated: bool  #: Activated.
    expected_blobs: int  #: Expected blobs.
    imported_blobs: int  #: Imported blobs.
    indexed_records: int  #: Indexed records.
    private_state_values: int  #: Private state values.
    repo_blocks: int  #: Repo blocks.
    repo_commit: string_formats.Cid  #: Repo commit.
    repo_rev: str  #: Repo rev.
    valid_did: bool  #: Valid did.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Data(base.DataModelBase):
    """Input data model for :obj:`tools.ozone.team.updateMember`."""

    did: string_formats.Did  #: Did.
    disabled: t.Optional[bool] = None  #: Disabled.
    role: t.Optional[
        t.Union[
            'models.ToolsOzoneTeamDefs.RoleAdmin',
            'models.ToolsOzoneTeamDefs.RoleModerator',
            'models.ToolsOzoneTeamDefs.RoleTriage',
            str,
        ]
    ] = None  #: Role.


class DataDict(t.TypedDict):
    did: string_formats.Did  #: Did.
    disabled: te.NotRequired[t.Optional[bool]]  #: Disabled.
    role: te.NotRequired[
        t.Optional[
            t.Union[
                'models.ToolsOzoneTeamDefs.RoleAdmin',
                'models.ToolsOzoneTeamDefs.RoleModerator',
                'models.ToolsOzoneTeamDefs.RoleTriage',
                str,
            ]
        ]
    ]  #: Role.



from atproto_core.exceptions import AtProtocolError


class InvalidTokenError(AtProtocolError):
    pass


class TokenDecodeError(InvalidTokenError):
    pass


class TokenInvalidSignatureError(TokenDecodeError):
    pass


class TokenInvalidAudienceError(InvalidTokenError):
    pass


class TokenExpiredSignatureError(InvalidTokenError):
    pass


class TokenInvalidIssuedAtError(InvalidTokenError):
    pass


class TokenImmatureSignatureError(InvalidTokenError):
    pass



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
    from atproto_client.models.unknown_type import UnknownType
from atproto_client.models import base


class StatusAttr(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    applied: bool  #: Applied.
    ref: t.Optional[str] = None  #: Ref.

    py_type: t.Literal['com.atproto.admin.defs#statusAttr'] = Field(
        default='com.atproto.admin.defs#statusAttr', alias='$type', frozen=True
    )


class AccountView(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    did: string_formats.Did  #: Did.
    handle: string_formats.Handle  #: Handle.
    indexed_at: string_formats.DateTime  #: Indexed at.
    deactivated_at: t.Optional[string_formats.DateTime] = None  #: Deactivated at.
    email: t.Optional[str] = None  #: Email.
    email_confirmed_at: t.Optional[string_formats.DateTime] = None  #: Email confirmed at.
    invite_note: t.Optional[str] = None  #: Invite note.
    invited_by: t.Optional['models.ComAtprotoServerDefs.InviteCode'] = None  #: Invited by.
    invites: t.Optional[t.List['models.ComAtprotoServerDefs.InviteCode']] = None  #: Invites.
    invites_disabled: t.Optional[bool] = None  #: Invites disabled.
    related_records: t.Optional[t.List['UnknownType']] = None  #: Related records.
    threat_signatures: t.Optional[t.List['models.ComAtprotoAdminDefs.ThreatSignature']] = None  #: Threat signatures.

    py_type: t.Literal['com.atproto.admin.defs#accountView'] = Field(
        default='com.atproto.admin.defs#accountView', alias='$type', frozen=True
    )


class RepoRef(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    did: string_formats.Did  #: Did.

    py_type: t.Literal['com.atproto.admin.defs#repoRef'] = Field(
        default='com.atproto.admin.defs#repoRef', alias='$type', frozen=True
    )


class RepoBlobRef(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    cid: string_formats.Cid  #: Cid.
    did: string_formats.Did  #: Did.
    record_uri: t.Optional[string_formats.AtUri] = None  #: Record uri.

    py_type: t.Literal['com.atproto.admin.defs#repoBlobRef'] = Field(
        default='com.atproto.admin.defs#repoBlobRef', alias='$type', frozen=True
    )


class ThreatSignature(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    property: str  #: Property.
    value: str  #: Value.

    py_type: t.Literal['com.atproto.admin.defs#threatSignature'] = Field(
        default='com.atproto.admin.defs#threatSignature', alias='$type', frozen=True
    )
